@@ -44,23 +44,6 @@ XMLSEC_CRYPTO_EXPORT void               xmlSecMSCryptoErrorsDefaultCallback(cons
                                                                         int reason,
                                                                         const char* msg);
 
-/******************************************************************************
- *
- * String encoding conversion utils
- *
- ******************************************************************************/
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT LPWSTR             xmlSecMSCryptoConvertLocaleToUnicode(const char* str);
-
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT LPWSTR             xmlSecMSCryptoConvertUtf8ToUnicode  (const xmlChar* str);
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT xmlChar*           xmlSecMSCryptoConvertUnicodeToUtf8  (LPCWSTR str);
-
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT xmlChar*           xmlSecMSCryptoConvertLocaleToUtf8   (const char* str);
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT char*              xmlSecMSCryptoConvertUtf8ToLocale   (const xmlChar* str);
-
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT xmlChar*           xmlSecMSCryptoConvertTstrToUtf8     (LPCTSTR str);
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT LPTSTR             xmlSecMSCryptoConvertUtf8ToTstr     (const xmlChar*  str);
-
-
 /********************************************************************
  *
  * DSA transform
@@ -203,7 +186,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaSha1GetKlass(vo
 /**
  * xmlSecMSCryptoTransformRsaSha256Id:
  *
- * The RSA-SHA256 signature transform klass.
+ * The RSA-SHA2-256 signature transform klass.
  */
 #define xmlSecMSCryptoTransformRsaSha256Id     \
        xmlSecMSCryptoTransformRsaSha256GetKlass()
@@ -214,7 +197,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaSha256GetKlass(
 /**
  * xmlSecMSCryptoTransformRsaSha384Id:
  *
- * The RSA-SHA384 signature transform klass.
+ * The RSA-SHA2-384 signature transform klass.
  */
 #define xmlSecMSCryptoTransformRsaSha384Id     \
        xmlSecMSCryptoTransformRsaSha384GetKlass()
@@ -225,7 +208,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaSha384GetKlass(
 /**
  * xmlSecMSCryptoTransformRsaSha512Id:
  *
- * The RSA-SHA512 signature transform klass.
+ * The RSA-SHA2-512 signature transform klass.
  */
 #define xmlSecMSCryptoTransformRsaSha512Id     \
        xmlSecMSCryptoTransformRsaSha512GetKlass()
@@ -244,22 +227,14 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaPkcs1GetKlass(v
 /**
  * xmlSecMSCryptoTransformRsaOaepId:
  *
- * The RSA OAEP key transport transform klass.
+ * The RSA OAEP key transport transform klass. MSCrypto only supports SHA1 for digest and MGF1.
  */
+#ifndef XMLSEC_NO_SHA1
 #define xmlSecMSCryptoTransformRsaOaepId \
         xmlSecMSCryptoTransformRsaOaepGetKlass()
 XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaOaepGetKlass(void);
+#endif /* XMLSEC_NO_SHA1 */
 
-/**
- * xmlSecMSCryptoTransformRsaOaepId:
- *
- * The RSA PKCS1 key transport transform klass.
- */
-/*
-#define xmlSecMSCryptoTransformRsaOaepId \
-        xmlSecMSCryptoTransformRsaOaepGetKlass()
-XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformRsaOaepGetKlass(void);
-*/
 #endif /* XMLSEC_NO_RSA */
 
 /********************************************************************
@@ -306,7 +281,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformSha1GetKlass(void)
 /**
  * xmlSecMSCryptoTransformSha256Id:
  *
- * The SHA256 digest transform klass.
+ * The SHA2-256 digest transform klass.
  */
 #define xmlSecMSCryptoTransformSha256Id \
        xmlSecMSCryptoTransformSha256GetKlass()
@@ -323,7 +298,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformSha256GetKlass(voi
 /**
  * xmlSecMSCryptoTransformSha384Id:
  *
- * The SHA384 digest transform klass.
+ * The SHA2-384 digest transform klass.
  */
 #define xmlSecMSCryptoTransformSha384Id \
        xmlSecMSCryptoTransformSha384GetKlass()
@@ -340,7 +315,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformSha384GetKlass(voi
 /**
  * xmlSecMSCryptoTransformSha512Id:
  *
- * The SHA512 digest transform klass.
+ * The SHA2-512 digest transform klass.
  */
 #define xmlSecMSCryptoTransformSha512Id \
        xmlSecMSCryptoTransformSha512GetKlass()
@@ -512,13 +487,10 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformKWDes3GetKlass(voi
  *******************************************************************/
 #ifndef XMLSEC_NO_HMAC
 
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT int  xmlSecMSCryptoHmacGetMinOutputLength(void);
-XMLSEC_DEPRECATED XMLSEC_CRYPTO_EXPORT void xmlSecMSCryptoHmacSetMinOutputLength(int min_length);
-
 /**
  * xmlSecMSCryptoKeyDataHmacId:
  *
- * The DHMAC key klass.
+ * The HMAC key klass.
  */
 #define xmlSecMSCryptoKeyDataHmacId \
         xmlSecMSCryptoKeyDataHmacGetKlass()
@@ -565,7 +537,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformHmacSha1GetKlass(v
 /**
  * xmlSecMSCryptoTransformHmacSha224Id:
  *
- * The HMAC with SHA224 signature transform klass.
+ * The HMAC with SHA2-224 signature transform klass.
  */
 #define xmlSecMSCryptoTransformHmacSha224Id \
         xmlSecMSCryptoTransformHmacSha224GetKlass()
@@ -576,7 +548,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformHmacSha224GetKlass
 /**
  * xmlSecMSCryptoTransformHmacSha256Id:
  *
- * The HMAC with SHA256 signature transform klass.
+ * The HMAC with SHA2-256 signature transform klass.
  */
 #define xmlSecMSCryptoTransformHmacSha256Id \
         xmlSecMSCryptoTransformHmacSha256GetKlass()
@@ -587,7 +559,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformHmacSha256GetKlass
 /**
  * xmlSecMSCryptoTransformHmacSha384Id:
  *
- * The HMAC with SHA384 signature transform klass.
+ * The HMAC with SHA2-384 signature transform klass.
  */
 #define xmlSecMSCryptoTransformHmacSha384Id \
         xmlSecMSCryptoTransformHmacSha384GetKlass()
@@ -598,7 +570,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCryptoTransformHmacSha384GetKlass
 /**
  * xmlSecMSCryptoTransformHmacSha512Id:
  *
- * The HMAC with SHA512 signature transform klass.
+ * The HMAC with SHA2-512 signature transform klass.
  */
 #define xmlSecMSCryptoTransformHmacSha512Id \
         xmlSecMSCryptoTransformHmacSha512GetKlass()
