@@ -2,6 +2,7 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates implementation for GnuTLS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
@@ -13,6 +14,7 @@
  * @Short_description: X509 certificates implementation for GnuTLS.
  * @Stability: Stable
  *
+ * X509 certificates implementation for GnuTLS.
  */
 
 #include "globals.h"
@@ -26,6 +28,10 @@
 #include <errno.h>
 #include <time.h>
 
+#include <gnutls/gnutls.h>
+#include <gnutls/abstract.h>
+#include <gnutls/x509.h>
+
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/base64.h>
 #include <xmlsec/keys.h>
@@ -39,18 +45,17 @@
 #include <xmlsec/gnutls/crypto.h>
 #include <xmlsec/gnutls/x509.h>
 
-#include "x509utils.h"
+#include "private.h"
 #include "../cast_helpers.h"
 #include "../keysdata_helpers.h"
 
-
 /*************************************************************************
  *
  * X509 utility functions
  *
  ************************************************************************/
-static int              xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data,
-                                                                 xmlSecKeyPtr key,
+static int              xmlSecGnuTLSKVerifyAndAdoptX509KeyData  (xmlSecKeyPtr key,
+                                                                 xmlSecKeyDataPtr data,
                                                                  xmlSecKeyInfoCtxPtr keyInfoCtx);
 
 /*************************************************************************
@@ -61,7 +66,7 @@ static int              xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(xmlSecKeyData
 typedef struct _xmlSecGnuTLSX509DataCtx                         xmlSecGnuTLSX509DataCtx,
                                                                 *xmlSecGnuTLSX509DataCtxPtr;
 struct _xmlSecGnuTLSX509DataCtx {
-    gnutls_x509_crt_t   keyCert;
+    gnutls_x509_crt_t   keyCert;    /* OWNED BY certsList */
     xmlSecPtrList       certsList;
     xmlSecPtrList       crlsList;
 };
@@ -69,7 +74,7 @@ struct _xmlSecGnuTLSX509DataCtx {
 
 /**************************************************************************
  *
- * <dsig:X509Data> processing (http://www.w3.org/TR/xmldsig-core/#sec-X509Data)
+ * &lt;dsig:X509Data/&gt; processing (http://www.w3.org/TR/xmldsig-core/#sec-X509Data)
  *
  *************************************************************************/
 XMLSEC_KEY_DATA_DECLARE(GnuTLSX509Data, xmlSecGnuTLSX509DataCtx)
@@ -104,11 +109,11 @@ typedef struct _xmlSecGnuTLSKeyDataX509Context {
 } xmlSecGnuTLSKeyDataX509Context;
 
 static int              xmlSecGnuTLSKeyDataX509Read             (xmlSecKeyDataPtr data,
-                                                                 xmlSecKeyValueX509Ptr x509Value,
+                                                                 xmlSecKeyX509DataValuePtr x509Value,
                                                                  xmlSecKeysMngrPtr keysMngr,
                                                                  unsigned int flags);
 static int              xmlSecGnuTLSKeyDataX509Write            (xmlSecKeyDataPtr data,
-                                                                  xmlSecKeyValueX509Ptr x509Value,
+                                                                  xmlSecKeyX509DataValuePtr x509Value,
                                                                   int content,
                                                                   void* context);
 
@@ -121,7 +126,7 @@ static xmlSecKeyDataKlass xmlSecGnuTLSKeyDataX509Klass = {
 
     /* data */
     xmlSecNameX509Data,
-    xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefX509Data,                         /* const xmlChar* href; */
     xmlSecNodeX509Data,                         /* const xmlChar* dataNodeName; */
@@ -186,18 +191,65 @@ xmlSecGnuTLSKeyDataX509GetKeyCert(xmlSecKeyDataPtr data) {
     return(ctx->keyCert);
 }
 
+static int
+xmlSecGnuTLSKeyDataX509AddCertInternal(xmlSecGnuTLSX509DataCtxPtr ctx, gnutls_x509_crt_t cert, int keyCert) {
+    gnutls_x509_crt_t cert2;
+    xmlSecSize ii, size;
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
+
+    /* we don't want duplicate certs */
+    size = xmlSecPtrListGetSize(&(ctx->certsList));
+    for(ii = 0; ii < size; ++ii) {
+        cert2 = xmlSecPtrListGetItem(&(ctx->certsList), ii);
+        if(cert2 == NULL) {
+            continue;
+        }
+        if((cert == cert2) || (gnutls_x509_crt_equals(cert, cert2) != 0)) {
+            ret = xmlSecPtrListRemove(&(ctx->certsList), ii);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecPtrListRemove()", NULL);
+                return(-1);
+            }
+            break;
+        }
+    }
+
+    /* ensure that key cert is the first one */
+    if(keyCert != 1) {
+        ret = xmlSecPtrListInsert(&(ctx->certsList), cert, 0);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecPtrListInsert(0)", NULL);
+            return(-1);
+        }
+    } else {
+        ret = xmlSecPtrListAdd(&(ctx->certsList), cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecPtrListAdd", NULL);
+            return(-1);
+        }
+    }
+
+    /* done */
+    return(0);
+}
+
 /**
  * xmlSecGnuTLSKeyDataX509AdoptKeyCert:
  * @data:               the pointer to X509 key data.
  * @cert:               the pointer to GnuTLS X509 certificate.
  *
- * Sets the key's certificate in @data.
+ * Adds certificate to the X509 key data and sets the it as the key's
+ * certificate in @data. On success, the @data owns the cert.
  *
  * Returns: 0 on success or a negative value if an error occurs.
  */
 int
 xmlSecGnuTLSKeyDataX509AdoptKeyCert(xmlSecKeyDataPtr data, gnutls_x509_crt_t cert) {
     xmlSecGnuTLSX509DataCtxPtr ctx;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGnuTLSKeyDataX509Id), -1);
     xmlSecAssert2(cert != NULL, -1);
@@ -205,9 +257,20 @@ xmlSecGnuTLSKeyDataX509AdoptKeyCert(xmlSecKeyDataPtr data, gnutls_x509_crt_t cer
     ctx = xmlSecGnuTLSX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, -1);
 
-    if(ctx->keyCert != NULL) {
-        gnutls_x509_crt_deinit(ctx->keyCert);
+    /* check if for some reasons same cert is used */
+    if((ctx->keyCert != NULL) && ((cert == ctx->keyCert) || (gnutls_x509_crt_equals(cert, ctx->keyCert) != 0))) {
+        gnutls_x509_crt_deinit(cert);  /* caller expects data to own the cert on success. */
+        return(0);
     }
+    xmlSecAssert2(ctx->keyCert == NULL, -1);
+
+    ret = xmlSecGnuTLSKeyDataX509AddCertInternal(ctx, cert, 1); /* key cert */
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSKeyDataX509AddCertInternal", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+
+    /* cert is now owned by data, we can't fail or there will be a double free */
     ctx->keyCert = cert;
     return(0);
 }
@@ -224,7 +287,6 @@ xmlSecGnuTLSKeyDataX509AdoptKeyCert(xmlSecKeyDataPtr data, gnutls_x509_crt_t cer
 int
 xmlSecGnuTLSKeyDataX509AdoptCert(xmlSecKeyDataPtr data, gnutls_x509_crt_t cert) {
     xmlSecGnuTLSX509DataCtxPtr ctx;
-    int ret;
 
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGnuTLSKeyDataX509Id), -1);
     xmlSecAssert2(cert != NULL, -1);
@@ -232,14 +294,12 @@ xmlSecGnuTLSKeyDataX509AdoptCert(xmlSecKeyDataPtr data, gnutls_x509_crt_t cert)
     ctx = xmlSecGnuTLSX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, -1);
 
-    ret = xmlSecPtrListAdd(&(ctx->certsList), cert);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecPtrListAdd",
-                            xmlSecKeyDataGetName(data));
-        return(-1);
+    /* pkcs12 files sometime have key cert twice: as the key cert and as the cert in the chain */
+    if((ctx->keyCert != NULL) && ((cert == ctx->keyCert) || (gnutls_x509_crt_equals(cert, ctx->keyCert) != 0))) {
+        gnutls_x509_crt_deinit(cert); /* caller expects data to own the cert on success. */
+        return(0);
     }
-
-    return(0);
+    return(xmlSecGnuTLSKeyDataX509AddCertInternal(ctx, cert, 0)); /* not a key cert */
 }
 
 /**
@@ -261,6 +321,8 @@ xmlSecGnuTLSKeyDataX509GetCert(xmlSecKeyDataPtr data, xmlSecSize pos) {
     ctx = xmlSecGnuTLSX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, NULL);
 
+    /* to ensure that key cert is always first we put it at the first position
+     * in xmlSecGnuTLSKeyDataX509AddCertInternal */
     return(xmlSecPtrListGetItem(&(ctx->certsList), pos));
 }
 
@@ -371,15 +433,13 @@ xmlSecGnuTLSKeyDataX509Initialize(xmlSecKeyDataPtr data) {
 
     ret = xmlSecPtrListInitialize(&(ctx->certsList), xmlSecGnuTLSX509CrtListId);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecPtrListInitialize(certsList)",
-                            xmlSecKeyDataGetName(data));
+        xmlSecInternalError("xmlSecPtrListInitialize(certsList)", NULL);
         return(-1);
     }
 
     ret = xmlSecPtrListInitialize(&(ctx->crlsList), xmlSecGnuTLSX509CrlListId);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecPtrListInitialize(crlsList)",
-                            xmlSecKeyDataGetName(data));
+        xmlSecInternalError("xmlSecPtrListInitialize(crlsList)", NULL);
         return(-1);
     }
 
@@ -396,30 +456,15 @@ xmlSecGnuTLSKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
     xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecGnuTLSKeyDataX509Id), -1);
 
     ctxSrc = xmlSecGnuTLSX509DataGetCtx(src);
-    xmlSecAssert2(ctxSrc != NULL, 0);
+    xmlSecAssert2(ctxSrc != NULL, -1);
     ctxDst = xmlSecGnuTLSX509DataGetCtx(dst);
-    xmlSecAssert2(ctxDst != NULL, 0);
-
-    /* copy key cert if exist */
-    if(ctxDst->keyCert != NULL) {
-        gnutls_x509_crt_deinit(ctxDst->keyCert);
-        ctxDst->keyCert = NULL;
-    }
-    if(ctxSrc->keyCert != NULL) {
-        ctxDst->keyCert = xmlSecGnuTLSX509CertDup(ctxSrc->keyCert);
-        if(ctxDst->keyCert == NULL) {
-            xmlSecInternalError("xmlSecGnuTLSX509CertDup",
-                                xmlSecKeyDataGetName(src));
-            return(-1);
-        }
-    }
+    xmlSecAssert2(ctxDst != NULL, -1);
 
     /* copy certsList if exists */
     xmlSecPtrListEmpty(&(ctxDst->certsList));
     ret = xmlSecPtrListCopy(&(ctxDst->certsList), &(ctxSrc->certsList));
     if(ret < 0) {
-        xmlSecInternalError("xmlSecPtrListCopy(certsList)",
-                            xmlSecKeyDataGetName(src));
+        xmlSecInternalError("xmlSecPtrListCopy(certsList)", NULL);
         return(-1);
     }
 
@@ -427,11 +472,28 @@ xmlSecGnuTLSKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
     xmlSecPtrListEmpty(&(ctxDst->crlsList));
     ret = xmlSecPtrListCopy(&(ctxDst->crlsList), &(ctxSrc->crlsList));
     if(ret < 0) {
-        xmlSecInternalError("xmlSecPtrListCopy(crlsList)",
-                            xmlSecKeyDataGetName(src));
+        xmlSecInternalError("xmlSecPtrListCopy(crlsList)", NULL);
         return(-1);
     }
 
+    /* keyCert: should be in the same position in certsList after copy */
+    if(ctxSrc->keyCert != NULL) {
+        xmlSecSize ii, size;
+
+        size = xmlSecPtrListGetSize(&(ctxDst->certsList));
+        xmlSecAssert2(size == xmlSecPtrListGetSize(&(ctxDst->certsList)), -1);
+        for(ii = 0; ii < size; ++ii) {
+            gnutls_x509_crt_t cert = xmlSecPtrListGetItem(&(ctxSrc->certsList), ii);
+            if(cert == ctxSrc->keyCert) {
+                ctxDst->keyCert = xmlSecPtrListGetItem(&(ctxDst->certsList), ii);
+                break;
+            }
+        }
+        /* just to double check */
+        xmlSecAssert2(ctxDst->keyCert != NULL, -1);
+        xmlSecAssert2(gnutls_x509_crt_equals(ctxSrc->keyCert, ctxDst->keyCert) != 0, -1);
+    }
+
     /* done */
     return(0);
 }
@@ -447,9 +509,7 @@ xmlSecGnuTLSKeyDataX509Finalize(xmlSecKeyDataPtr data) {
 
     xmlSecPtrListFinalize(&(ctx->crlsList));
     xmlSecPtrListFinalize(&(ctx->certsList));
-    if(ctx->keyCert != NULL) {
-        gnutls_x509_crt_deinit(ctx->keyCert);
-    }
+
     memset(ctx, 0, sizeof(xmlSecGnuTLSX509DataCtx));
 }
 
@@ -462,27 +522,38 @@ xmlSecGnuTLSKeyDataX509XmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
     xmlSecAssert2(id == xmlSecGnuTLSKeyDataX509Id, -1);
     xmlSecAssert2(key != NULL, -1);
 
-    data = xmlSecKeyEnsureData(key, id);
+    data = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataX509Id);
     if(data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataX509Id)", xmlSecKeyDataKlassGetName(id));
         return(-1);
     }
 
-    ret = xmlSecKeyDataX509XmlRead(data, node, keyInfoCtx,
-        xmlSecGnuTLSKeyDataX509Read);
+    ret = xmlSecKeyDataX509XmlRead(key, data, node, keyInfoCtx, xmlSecGnuTLSKeyDataX509Read);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecKeyDataX509XmlRead",
-            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataX509XmlRead", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
         return(-1);
     }
 
-    ret = xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(data, key, keyInfoCtx);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecGnuTLSKeyDataX509VerifyAndExtractKey",
-            xmlSecKeyDataKlassGetName(id));
-        return(-1);
+    /* did we find the key already? */
+    if(xmlSecKeyGetValue(key) != NULL) {
+        xmlSecKeyDataDestroy(data);
+        return(0);
     }
+
+    ret = xmlSecGnuTLSKVerifyAndAdoptX509KeyData(key, data, keyInfoCtx);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSKVerifyAndAdoptX509KeyData", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
+        return(-1);
+    } else if(ret != 1) {
+        /* no errors but key was not found and data was not adopted */
+        xmlSecKeyDataDestroy(data);
+        return(0);
+    }
+    data = NULL; /* owned by data now */
+
+    /* success */
     return(0);
 }
 
@@ -652,13 +723,11 @@ xmlSecGnuTLSKeyDataX509DebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
     fprintf(output, "</X509Data>\n");
 }
 
-
+/* xmlSecKeyDataX509Read: 0 on success and a negative value otherwise */
 static int
-xmlSecGnuTLSKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Value,
+xmlSecGnuTLSKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr x509Value,
                              xmlSecKeysMngrPtr keysMngr, unsigned int flags) {
     xmlSecKeyDataStorePtr x509Store;
-    int stopOnUnknownCert = 0;
-    gnutls_x509_crt_t storeCert = NULL;
     gnutls_x509_crt_t cert = NULL;
     gnutls_x509_crl_t crl = NULL;
     int ret;
@@ -675,11 +744,6 @@ xmlSecGnuTLSKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
         goto done;
     }
 
-    /* determine what to do */
-    if((flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_UNKNOWN_CERT) != 0) {
-        stopOnUnknownCert = 1;
-    }
-
     if(xmlSecBufferGetSize(&(x509Value->cert)) > 0) {
         cert = xmlSecGnuTLSX509CertRead(xmlSecBufferGetData(&(x509Value->cert)),
             xmlSecBufferGetSize(&(x509Value->cert)), xmlSecKeyDataFormatCertDer);
@@ -687,50 +751,39 @@ xmlSecGnuTLSKeyDataX509Read(xmlSecKeyDataPtr data, xmlSecKeyValueX509Ptr x509Val
             xmlSecInternalError("xmlSecGnuTLSX509CertRead", xmlSecKeyDataGetName(data));
             goto done;
         }
-    } else if(xmlSecBufferGetSize(&(x509Value->crl)) > 0) {
+    }
+
+    if(xmlSecBufferGetSize(&(x509Value->crl)) > 0) {
         crl = xmlSecGnuTLSX509CrlRead(xmlSecBufferGetData(&(x509Value->crl)),
             xmlSecBufferGetSize(&(x509Value->crl)), xmlSecKeyDataFormatCertDer);
         if(crl == NULL) {
             xmlSecInternalError("xmlSecGnuTLSX509CrlRead", xmlSecKeyDataGetName(data));
             goto done;
         }
-    } else if(xmlSecBufferGetSize(&(x509Value->ski)) > 0) {
-        storeCert = xmlSecGnuTLSX509StoreFindCert_ex(x509Store, NULL,  NULL, NULL,
-            xmlSecBufferGetData(&(x509Value->ski)), xmlSecBufferGetSize(&(x509Value->ski)),
-            NULL /* unused */);
-        if((storeCert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "skiSize=" XMLSEC_SIZE_FMT, xmlSecBufferGetSize(&(x509Value->ski)));
-            goto done;
-        }
-    } else if(x509Value->subject != NULL) {
-        storeCert = xmlSecGnuTLSX509StoreFindCert_ex(x509Store, x509Value->subject,
-            NULL, NULL, NULL, 0, NULL /* unused */);
-        if((storeCert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError2(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "subject=%s", xmlSecErrorsSafeString(x509Value->subject));
-            goto done;
-        }
-    } else if((x509Value->issuerName != NULL) && (x509Value->issuerSerial != NULL)) {
-        storeCert = xmlSecGnuTLSX509StoreFindCert_ex(x509Store, NULL,
-            x509Value->issuerName, x509Value->issuerSerial,
-            NULL, 0, NULL /* unused */);
-        if((storeCert == NULL) && (stopOnUnknownCert != 0)) {
-            xmlSecOtherError3(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data),
-                "issuerName=%s;issuerSerial=%s",
-                xmlSecErrorsSafeString(x509Value->issuerName),
-                xmlSecErrorsSafeString(x509Value->issuerSerial));
-            goto done;
-        }
     }
 
-    /* if we found cert in a store, then duplicate it for key data */
-    if((cert == NULL) && (storeCert != NULL)) {
-        cert = xmlSecGnuTLSX509CertDup(storeCert);
-        if(cert == NULL) {
-            xmlSecInternalError("xmlSecGnuTLSX509CertDup", xmlSecKeyDataGetName(data));
+    /* if there is no cert in the X509Data node then try to find one */
+    if(cert == NULL) {
+        gnutls_x509_crt_t storeCert = NULL;
+        int stopOnUnknownCert = 0;
+
+        /* determine what to do */
+        if((flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_UNKNOWN_CERT) != 0) {
+            stopOnUnknownCert = 1;
+        }
+        storeCert = xmlSecGnuTLSX509StoreFindCertByValue(x509Store, x509Value);
+        if((storeCert == NULL) && (stopOnUnknownCert != 0)) {
+            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data), "cert lookup");
             goto done;
         }
+        /* if we found cert in a store, then duplicate it for key data */
+        if(storeCert != NULL) {
+            cert = xmlSecGnuTLSX509CertDup(storeCert);
+            if(cert == NULL) {
+                xmlSecInternalError("xmlSecGnuTLSX509CertDup", xmlSecKeyDataGetName(data));
+                goto done;
+            }
+        }
     }
 
     /* if we found a cert or a crl, then add it to the data */
@@ -765,8 +818,11 @@ done:
     return(res);
 }
 
+/* xmlSecKeyDataX509Write: returns 1 on success, 0 if no more certs/crls are available,
+ * or a negative value if an error occurs.
+ */
 static int
-xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509Value,
+xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyX509DataValuePtr x509Value,
                             int content, void* context) {
     xmlSecGnuTLSKeyDataX509Context* ctx;
     int ret;
@@ -786,7 +842,7 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
                 "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
             return(-1);
         }
-        if((content & XMLSEC_X509DATA_CERTIFICATE_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_CERTIFICATE_NODE)) {
             ret = xmlSecGnuTLSX509CertDerWrite(cert, &(x509Value->cert));
             if(ret < 0) {
                 xmlSecInternalError2("xmlSecGnuTLSX509CertDerWrite",
@@ -795,7 +851,7 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
                 return(-1);
             }
         }
-        if((content & XMLSEC_X509DATA_SKI_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SKI_NODE)) {
             ret = xmlSecGnuTLSX509CertSKIWrite(cert, &(x509Value->ski));
             if(ret < 0) {
                 xmlSecInternalError2("xmlSecGnuTLSX509SKIWrite",
@@ -804,7 +860,7 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
                 return(-1);
             }
         }
-        if((content & XMLSEC_X509DATA_SUBJECTNAME_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_SUBJECTNAME_NODE)) {
             xmlSecAssert2(x509Value->subject == NULL, -1);
 
             x509Value->subject = xmlSecGnuTLSX509CertGetSubjectDN(cert);
@@ -815,7 +871,7 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
                 return(-1);
             }
         }
-        if((content & XMLSEC_X509DATA_ISSUERSERIAL_NODE) != 0) {
+        if (XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_ISSUERSERIAL_NODE)) {
             xmlSecAssert2(x509Value->issuerName == NULL, -1);
             xmlSecAssert2(x509Value->issuerSerial == NULL, -1);
 
@@ -834,6 +890,15 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
                 return(-1);
             }
         }
+        if((XMLSEC_X509DATA_HAS_EMPTY_NODE(content, XMLSEC_X509DATA_DIGEST_NODE)) && (x509Value->digestAlgorithm != NULL)) {
+            ret = xmlSecGnuTLSX509DigestWrite(cert, x509Value->digestAlgorithm, &(x509Value->digest));
+            if(ret < 0) {
+                xmlSecInternalError2("xmlSecGnuTLSX509DigestWrite",
+                    xmlSecKeyDataGetName(data),
+                    "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
+                return(-1);
+            }
+        }
         ++ctx->crtPos;
     } else if(ctx->crlPos < ctx->crlSize) {
         /* write crl */
@@ -857,11 +922,11 @@ xmlSecGnuTLSKeyDataX509Write(xmlSecKeyDataPtr data,  xmlSecKeyValueX509Ptr x509V
         ++ctx->crlPos;
     } else {
         /* no more certs or crls */
-        return(1);
+        return(0);
     }
 
     /* success */
-    return(0);
+    return(1);
 }
 
 static int
@@ -906,10 +971,12 @@ xmlSecGnuTLSX509CertSKIWrite(gnutls_x509_crt_t cert, xmlSecBufferPtr buf) {
 }
 
 static int
-xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data, xmlSecKeyPtr key,
-                                    xmlSecKeyInfoCtxPtr keyInfoCtx) {
+xmlSecGnuTLSKVerifyAndAdoptX509KeyData(xmlSecKeyPtr key, xmlSecKeyDataPtr data,  xmlSecKeyInfoCtxPtr keyInfoCtx) {
     xmlSecGnuTLSX509DataCtxPtr ctx;
     xmlSecKeyDataStorePtr x509Store;
+    gnutls_x509_crt_t cert;
+    gnutls_x509_crt_t keyCert;
+    xmlSecKeyDataPtr keyValue;
     int ret;
 
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGnuTLSKeyDataX509Id), -1);
@@ -919,74 +986,87 @@ xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data, xmlSecKeyPtr k
 
     ctx = xmlSecGnuTLSX509DataGetCtx(data);
     xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->keyCert == NULL, -1);
 
-    x509Store = xmlSecKeysMngrGetDataStore(keyInfoCtx->keysMngr, xmlSecGnuTLSX509StoreId);
-    if(x509Store == NULL) {
-        xmlSecInternalError("xmlSecKeysMngrGetDataStore",
-                            xmlSecKeyDataGetName(data));
-        return(-1);
+    if( (xmlSecPtrListGetSize(&(ctx->certsList)) <= 0) || (xmlSecKeyGetValue(key) != NULL)) {
+        /* no certs or key was already found -> nothing to do (this shouldn't really happen) */
+        return(0);
     }
 
-    if((ctx->keyCert == NULL) && (xmlSecPtrListGetSize(&(ctx->certsList)) > 0) && (xmlSecKeyGetValue(key) == NULL)) {
-        gnutls_x509_crt_t cert;
-
-        cert = xmlSecGnuTLSX509StoreVerify(x509Store, &(ctx->certsList), &(ctx->crlsList), keyInfoCtx);
-        if(cert != NULL) {
-            xmlSecKeyDataPtr keyValue;
-
-            ctx->keyCert = xmlSecGnuTLSX509CertDup(cert);
-            if(ctx->keyCert == NULL) {
-                xmlSecInternalError("xmlSecGnuTLSX509CertDup",
-                                    xmlSecKeyDataGetName(data));
-                return(-1);
-            }
-
-            keyValue = xmlSecGnuTLSX509CertGetKey(ctx->keyCert);
-            if(keyValue == NULL) {
-                xmlSecInternalError("xmlSecGnuTLSX509CertGetKey",
-                                    xmlSecKeyDataGetName(data));
-                return(-1);
-            }
-
-            /* verify that the key matches our expectations */
-            if(xmlSecKeyReqMatchKeyValue(&(keyInfoCtx->keyReq), keyValue) != 1) {
-                xmlSecInternalError("xmlSecKeyReqMatchKeyValue",
-                                    xmlSecKeyDataGetName(data));
-                xmlSecKeyDataDestroy(keyValue);
-                return(-1);
-            }
-
-            ret = xmlSecKeySetValue(key, keyValue);
-            if(ret < 0) {
-                xmlSecInternalError("xmlSecKeySetValue",
-                                    xmlSecKeyDataGetName(data));
-                xmlSecKeyDataDestroy(keyValue);
-                return(-1);
-            }
-
-            /* get expiration time */
-            key->notValidBefore = gnutls_x509_crt_get_activation_time(ctx->keyCert);
-            if(key->notValidBefore == (time_t)-1) {
-                xmlSecGnuTLSError2("gnutls_x509_crt_get_activation_time", GNUTLS_E_SUCCESS,
-                    xmlSecKeyDataGetName(data),
-                    "cert activation time is invalid: %.lf",
-                    difftime(key->notValidBefore, (time_t)0));
-                return(-1);
-            }
-            key->notValidAfter = gnutls_x509_crt_get_expiration_time(ctx->keyCert);
-            if(key->notValidAfter == (time_t)-1) {
-                xmlSecGnuTLSError2("gnutls_x509_crt_get_expiration_time", GNUTLS_E_SUCCESS,
-                    xmlSecKeyDataGetName(data),
-                    "cert expiration time is invalid: %.lf",
-                    difftime(key->notValidAfter, (time_t)0));
-                return(-1);
-            }
-        } else if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_INVALID_CERT) != 0) {
+    /* lets find a cert we can verify */
+    x509Store = xmlSecKeysMngrGetDataStore(keyInfoCtx->keysMngr, xmlSecGnuTLSX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    cert = xmlSecGnuTLSX509StoreVerify(x509Store, &(ctx->certsList), &(ctx->crlsList), keyInfoCtx);
+    if(cert == NULL) {
+        /* check if we want to fail if cert is not found */
+        if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_STOP_ON_INVALID_CERT) != 0) {
             xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND, xmlSecKeyDataGetName(data), NULL);
             return(-1);
         }
+        return(0);
     }
-    return(0);
+
+    /* set cert into the x509 data, we don't know if the cert is already in KeyData or not
+     * so assume we need to add it again.
+     */
+    keyCert = xmlSecGnuTLSX509CertDup(cert);
+    if(keyCert == NULL) {
+        xmlSecInternalError("xmlSecGnuTLSX509CertDup", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    ret = xmlSecGnuTLSKeyDataX509AdoptKeyCert(data, keyCert);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSKeyDataX509AdoptKeyCert", xmlSecKeyDataGetName(data));
+        gnutls_x509_crt_deinit(keyCert);
+        return(-1);
+    }
+    cert = keyCert = NULL; /* we should be using ctx->keyCert for everything */
+
+    /* extract key from cert and verify that the key matches our expectations */
+    keyValue = xmlSecGnuTLSX509CertGetKey(ctx->keyCert);
+    if(keyValue == NULL) {
+        xmlSecInternalError("xmlSecGnuTLSX509CertGetKey", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    if(xmlSecKeyReqMatchKeyValue(&(keyInfoCtx->keyReq), keyValue) != 1) {
+        xmlSecInternalError("xmlSecKeyReqMatchKeyValue", xmlSecKeyDataGetName(data));
+        xmlSecKeyDataDestroy(keyValue);
+        return(-1);
+    }
+    ret = xmlSecKeySetValue(key, keyValue);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecKeySetValue", xmlSecKeyDataGetName(data));
+        xmlSecKeyDataDestroy(keyValue);
+        return(-1);
+    }
+    keyValue = NULL; /* owned by key now */
+
+    /* copy cert not before / not after times from the cert */
+    key->notValidBefore = gnutls_x509_crt_get_activation_time(ctx->keyCert);
+    if(key->notValidBefore == (time_t)-1) {
+        xmlSecGnuTLSError("gnutls_x509_crt_get_activation_time", GNUTLS_E_SUCCESS, xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+    key->notValidAfter = gnutls_x509_crt_get_expiration_time(ctx->keyCert);
+    if(key->notValidAfter == (time_t)-1) {
+        xmlSecGnuTLSError("gnutls_x509_crt_get_expiration_time", GNUTLS_E_SUCCESS, xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+
+    /* THIS MUST BE THE LAST THING WE DO: add data to the key
+     * if we do it sooner and fail later then both the caller and the key will free data
+     * which would lead to double free */
+    ret = xmlSecKeyAdoptData(key, data);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecKeyAdoptData", xmlSecKeyDataGetName(data));
+        return(-1);
+    }
+
+    /* success: cert found and data was adopted */
+    return(1);
 }
 
 /**
@@ -1000,88 +1080,33 @@ xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data, xmlSecKeyPtr k
 xmlSecKeyDataPtr
 xmlSecGnuTLSX509CertGetKey(gnutls_x509_crt_t cert) {
     xmlSecKeyDataPtr data;
-    int alg;
-    unsigned int bits;
+    gnutls_pubkey_t pubkey;
     int err;
-    int ret;
 
     xmlSecAssert2(cert != NULL, NULL);
 
-    alg = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
-    if(alg < 0) {
-        xmlSecGnuTLSError("gnutls_x509_crt_get_pk_algorithm", alg, NULL);
+    err = gnutls_pubkey_init(&pubkey);
+    if (err != GNUTLS_E_SUCCESS) {
+        xmlSecGnuTLSError("gnutls_pubkey_init", err, NULL);
         return(NULL);
     }
 
-    switch(alg) {
-#ifndef XMLSEC_NO_RSA
-    case GNUTLS_PK_RSA:
-        {
-            gnutls_datum_t m, e;
-
-            data = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataRsaId);
-            if(data == NULL) {
-                xmlSecInternalError("xmlSecKeyDataCreate(KeyDataRsaId)", NULL);
-                return(NULL);
-            }
-
-            err = gnutls_x509_crt_get_pk_rsa_raw(cert, &m, &e);
-            if(err != GNUTLS_E_SUCCESS) {
-                xmlSecGnuTLSError("gnutls_x509_crt_get_pk_rsa_raw", err, NULL);
-                return(NULL);
-            }
-
-            ret = xmlSecGnuTLSKeyDataRsaAdoptPublicKey(data, &m, &e);
-            if(ret < 0) {
-                xmlSecInternalError("xmlSecGnuTLSKeyDataRsaAdoptPublicKey", NULL);
-                gnutls_free(m.data);
-                gnutls_free(e.data);
-                return(NULL);
-            }
-            /* m and e are owned by data now */
-        }
-        break;
-#endif /* XMLSEC_NO_RSA */
-
-#ifndef XMLSEC_NO_DSA
-    case GNUTLS_PK_DSA:
-        {
-            gnutls_datum_t p, q, g, y;
-
-            data = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataDsaId);
-            if(data == NULL) {
-                xmlSecInternalError("xmlSecKeyDataCreate(KeyDataDsaId)", NULL);
-                return(NULL);
-            }
-
-            err = gnutls_x509_crt_get_pk_dsa_raw(cert, &p, &q, &g, &y);
-            if(err != GNUTLS_E_SUCCESS) {
-                xmlSecGnuTLSError("gnutls_x509_crt_get_pk_dsa_raw", err, NULL);
-                return(NULL);
-            }
-
-            ret = xmlSecGnuTLSKeyDataDsaAdoptPublicKey(data, &p, &q, &g, &y);
-            if(ret < 0) {
-                xmlSecInternalError("xmlSecGnuTLSKeyDataDsaAdoptPublicKey", NULL);
-                gnutls_free(p.data);
-                gnutls_free(q.data);
-                gnutls_free(g.data);
-                gnutls_free(y.data);
-                return(NULL);
-            }
-            /* p, q, g and y are owned by data now */
-        }
-        break;
-#endif /* XMLSEC_NO_DSA */
-
-    default:
-        {
-            xmlSecInvalidIntegerTypeError("key_alg", alg, "supported algorithm", NULL);
-            return(NULL);
-        }
+    err = gnutls_pubkey_import_x509(pubkey, cert, 0);
+    if (err != GNUTLS_E_SUCCESS) {
+        xmlSecGnuTLSError("gnutls_pubkey_import_x509", err, NULL);
+        gnutls_pubkey_deinit(pubkey);
+        return(NULL);
     }
 
-    /* data */
+    data = xmlSecGnuTLSAsymKeyDataCreate(pubkey, NULL);
+    if(data == NULL) {
+        xmlSecInternalError("xmlSecGnuTLSAsymKeyDataCreate", NULL);
+        gnutls_pubkey_deinit(pubkey);
+        return(NULL);
+    }
+    pubkey = NULL; /* owned by data */
+
+    /* done */
     return(data);
 }
 
@@ -1104,7 +1129,7 @@ static xmlSecKeyDataKlass xmlSecGnuTLSKeyDataRawX509CertKlass = {
 
     /* data */
     xmlSecNameRawX509Cert,
-    xmlSecKeyDataUsageRetrievalMethodNodeBin,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageRetrievalMethodNodeBin,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefRawX509Cert,                      /* const xmlChar* href; */
     NULL,                                       /* const xmlChar* dataNodeName; */
@@ -1168,28 +1193,35 @@ xmlSecGnuTLSKeyDataRawX509CertBinRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
         return(-1);
     }
 
-    data = xmlSecKeyEnsureData(key, xmlSecGnuTLSKeyDataX509Id);
+    data = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataX509Id);
     if(data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData",
-                            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataX509Id)", xmlSecKeyDataKlassGetName(id));
         gnutls_x509_crt_deinit(cert);
         return(-1);
     }
 
     ret = xmlSecGnuTLSKeyDataX509AdoptCert(data, cert);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecGnuTLSKeyDataX509AdoptCert",
-                            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecGnuTLSKeyDataX509AdoptCert", xmlSecKeyDataKlassGetName(id));
         gnutls_x509_crt_deinit(cert);
+        xmlSecKeyDataDestroy(data);
         return(-1);
     }
+    cert = NULL; /* owned by data now */
 
-    ret = xmlSecGnuTLSKeyDataX509VerifyAndExtractKey(data, key, keyInfoCtx);
+    ret = xmlSecGnuTLSKVerifyAndAdoptX509KeyData(key, data, keyInfoCtx);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecGnuTLSKeyDataX509VerifyAndExtractKey",
-                            xmlSecKeyDataKlassGetName(id));
+        xmlSecInternalError("xmlSecGnuTLSKVerifyAndAdoptX509KeyData", xmlSecKeyDataKlassGetName(id));
+        xmlSecKeyDataDestroy(data);
         return(-1);
+    } else if(ret != 1) {
+        /* no errors but key was not found and data was not adopted */
+        xmlSecKeyDataDestroy(data);
+        return(0);
     }
+    data = NULL; /* owned by data now */
+
+    /* success */
     return(0);
 }
 
