@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * X509 certificates verification support functions for GnuTLS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:x509vfy
- * @Short_description: X509 certificates verification support functions for GnuTLS.
- * @Stability: Private
- *
+ * SECTION:x509
  */
 
 #include "globals.h"
@@ -35,7 +33,7 @@
 #include <xmlsec/gnutls/crypto.h>
 #include <xmlsec/gnutls/x509.h>
 
-#include "x509utils.h"
+#include "private.h"
 #include "../cast_helpers.h"
 
 /**************************************************************************
@@ -48,6 +46,7 @@ typedef struct _xmlSecGnuTLSX509StoreCtx                xmlSecGnuTLSX509StoreCtx
 struct _xmlSecGnuTLSX509StoreCtx {
     xmlSecPtrList certsTrusted;
     xmlSecPtrList certsUntrusted;
+    xmlSecPtrList crls;
 };
 
 /****************************************************************************
@@ -78,20 +77,13 @@ static xmlSecKeyDataStoreKlass xmlSecGnuTLSX509StoreKlass = {
 };
 
 static gnutls_x509_crt_t xmlSecGnuTLSX509FindCert                       (xmlSecPtrListPtr certs,
-                                                                         const xmlChar *subjectName,
-                                                                         const xmlChar *issuerName,
-                                                                         const xmlChar *issuerSerial,
-                                                                         const xmlSecByte * ski,
-                                                                         xmlSecSize skiSize);
+                                                                         xmlSecGnuTLSX509FindCertCtxPtr findCertCtx);
 static gnutls_x509_crt_t xmlSecGnuTLSX509FindSignedCert                 (xmlSecPtrListPtr certs,
                                                                          gnutls_x509_crt_t cert);
 static gnutls_x509_crt_t xmlSecGnuTLSX509FindSignerCert                 (xmlSecPtrListPtr certs,
                                                                          gnutls_x509_crt_t cert);
 
 
-static int               xmlSecGnuTLSX509CertCompareSKI                 (gnutls_x509_crt_t cert,
-                                                                         const xmlSecByte * ski,
-                                                                         xmlSecSize skiSize);
 /**
  * xmlSecGnuTLSX509StoreGetKlass:
  *
@@ -111,7 +103,7 @@ xmlSecGnuTLSX509StoreGetKlass(void) {
  * @issuerName:         the desired certificate issuer name.
  * @issuerSerial:       the desired certificate issuer serial number.
  * @ski:                the desired certificate SKI.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -162,7 +154,7 @@ xmlSecGnuTLSX509StoreFindCert(const xmlSecKeyDataStorePtr store, const xmlChar *
  * @issuerSerial:       the desired certificate issuer serial number.
  * @ski:                the desired certificate SKI.
  * @skiSize:            the desired certificate SKI size.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Searches @store for a certificate that matches given criteria.
  *
@@ -175,7 +167,9 @@ xmlSecGnuTLSX509StoreFindCert_ex(const xmlSecKeyDataStorePtr store, const xmlCha
                               const xmlSecByte * ski, xmlSecSize skiSize,
                               const xmlSecKeyInfoCtx* keyInfoCtx ATTRIBUTE_UNUSED) {
     xmlSecGnuTLSX509StoreCtxPtr ctx;
+    xmlSecGnuTLSX509FindCertCtx findCertCtx;
     gnutls_x509_crt_t res = NULL;
+    int ret;
 
     xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecGnuTLSX509StoreId), NULL);
     UNREFERENCED_PARAMETER(keyInfoCtx);
@@ -183,26 +177,156 @@ xmlSecGnuTLSX509StoreFindCert_ex(const xmlSecKeyDataStorePtr store, const xmlCha
     ctx = xmlSecGnuTLSX509StoreGetCtx(store);
     xmlSecAssert2(ctx != NULL, NULL);
 
-    if(res == NULL) {
-        res = xmlSecGnuTLSX509FindCert(&(ctx->certsTrusted), subjectName,
+    ret = xmlSecGnuTLSX509FindCertCtxInitialize(&findCertCtx,
+            subjectName,
             issuerName, issuerSerial,
             ski, skiSize);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSX509FindCertCtxInitialize", NULL);
+        xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    if(res == NULL) {
+        res = xmlSecGnuTLSX509FindCert(&(ctx->certsTrusted), &findCertCtx);
     }
     if(res == NULL) {
-        res = xmlSecGnuTLSX509FindCert(&(ctx->certsUntrusted), subjectName,
-            issuerName, issuerSerial,
-            ski, skiSize);
+        res = xmlSecGnuTLSX509FindCert(&(ctx->certsUntrusted), &findCertCtx);
     }
+
+    /* done */
+    xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+    return(res);
+}
+
+gnutls_x509_crt_t
+xmlSecGnuTLSX509StoreFindCertByValue(xmlSecKeyDataStorePtr store, xmlSecKeyX509DataValuePtr x509Value) {
+    xmlSecGnuTLSX509StoreCtxPtr ctx;
+    xmlSecGnuTLSX509FindCertCtx findCertCtx;
+    int ret;
+    gnutls_x509_crt_t res = NULL;
+
+    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecGnuTLSX509StoreId), NULL);
+
+    ctx = xmlSecGnuTLSX509StoreGetCtx(store);
+    xmlSecAssert2(ctx != NULL, NULL);
+
+    ret = xmlSecGnuTLSX509FindCertCtxInitializeFromValue(&findCertCtx, x509Value);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSX509FindCertCtxInitializeFromValue", NULL);
+        xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    if(res == NULL) {
+        res = xmlSecGnuTLSX509FindCert(&(ctx->certsTrusted), &findCertCtx);
+    }
+    if(res == NULL) {
+        res = xmlSecGnuTLSX509FindCert(&(ctx->certsUntrusted), &findCertCtx);
+    }
+
+    /* done */
+    xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+    return(res);
+}
+
+xmlSecKeyPtr
+xmlSecGnuTLSX509FindKeyByValue(xmlSecPtrListPtr keysList, xmlSecKeyX509DataValuePtr x509Value) {
+    xmlSecGnuTLSX509FindCertCtx findCertCtx;
+    xmlSecSize keysListSize, ii;
+    xmlSecKeyPtr res = NULL;
+    int ret;
+
+    xmlSecAssert2(keysList != NULL, NULL);
+    xmlSecAssert2(x509Value != NULL, NULL);
+
+    ret = xmlSecGnuTLSX509FindCertCtxInitializeFromValue(&findCertCtx, x509Value);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSX509FindCertCtxInitializeFromValue", NULL);
+        xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+        return(NULL);
+    }
+
+    keysListSize = xmlSecPtrListGetSize(keysList);
+    for(ii = 0; ii < keysListSize; ++ii) {
+        xmlSecKeyPtr key;
+        xmlSecKeyDataPtr keyData;
+        gnutls_x509_crt_t keyCert;
+
+        /* get key's cert from x509 key data */
+        key = (xmlSecKeyPtr)xmlSecPtrListGetItem(keysList, ii);
+        if(key == NULL) {
+            continue;
+        }
+        keyData = xmlSecKeyGetData(key, xmlSecGnuTLSKeyDataX509Id);
+        if(keyData == NULL) {
+            continue;
+        }
+        keyCert = xmlSecGnuTLSKeyDataX509GetKeyCert(keyData);
+        if(keyCert == NULL) {
+            continue;
+        }
+
+        /* does it match? */
+        ret = xmlSecGnuTLSX509FindCertCtxMatch(&findCertCtx, keyCert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecGnuTLSX509FindCertCtxMatch", NULL);
+            xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
+            return(NULL);
+        } else if(ret == 1) {
+            res = key;
+            break;
+        }
+    }
+
+    /* done */
+    xmlSecGnuTLSX509FindCertCtxFinalize(&findCertCtx);
     return(res);
 }
 
 static int
-xmlSecGnuTLSX509CheckTime(const gnutls_x509_crt_t * cert_list,
-                          xmlSecSize cert_list_size,
-                          time_t ts)
-{
+xmlSecGnuTLSX509CheckCrtTime(const gnutls_x509_crt_t cert, time_t ts) {
     time_t notValidBefore, notValidAfter;
+
+    xmlSecAssert2(cert != NULL, -1);
+
+    /* get expiration times */
+    notValidBefore = gnutls_x509_crt_get_activation_time(cert);
+    if(notValidBefore == (time_t)-1) {
+        xmlSecGnuTLSError2("gnutls_x509_crt_get_activation_time", GNUTLS_E_SUCCESS, NULL,
+            "cert activation time is invalid: %.lf",
+            difftime(notValidBefore, (time_t)0));
+        return(-1);
+    }
+    notValidAfter = gnutls_x509_crt_get_expiration_time(cert);
+    if(notValidAfter == (time_t)-1) {
+        xmlSecGnuTLSError2("gnutls_x509_crt_get_expiration_time", GNUTLS_E_SUCCESS, NULL,
+            "cert expiration time is invalid: %.lf",
+            difftime(notValidAfter, (time_t)0));
+        return(-1);
+    }
+
+    /* check */
+    if(ts < notValidBefore) {
+        /* TODO: print cert subject */
+        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_YET_VALID, NULL, NULL);
+        return(0);
+    }
+    if(ts > notValidAfter) {
+        /* TODO: print cert subject */
+        xmlSecOtherError(XMLSEC_ERRORS_R_CERT_HAS_EXPIRED, NULL, NULL);
+        return(0);
+    }
+
+    /* good! */
+    return(1);
+}
+
+
+static int
+xmlSecGnuTLSX509CheckCrtsTime(const gnutls_x509_crt_t * cert_list, xmlSecSize cert_list_size, time_t ts) {
     xmlSecSize ii;
+    int ret;
 
     xmlSecAssert2(cert_list != NULL, -1);
 
@@ -212,31 +336,12 @@ xmlSecGnuTLSX509CheckTime(const gnutls_x509_crt_t * cert_list,
             continue;
         }
 
-        /* get expiration times */
-        notValidBefore = gnutls_x509_crt_get_activation_time(cert);
-        if(notValidBefore == (time_t)-1) {
-            xmlSecGnuTLSError2("gnutls_x509_crt_get_activation_time", GNUTLS_E_SUCCESS,
-                NULL,
-                "cert activation time is invalid: %.lf",
-                difftime(notValidBefore, (time_t)0));
+        ret = xmlSecGnuTLSX509CheckCrtTime(cert, ts);
+        if(ret < 0) {
+            xmlSecInternalError("", NULL);
             return(-1);
-        }
-        notValidAfter = gnutls_x509_crt_get_expiration_time(cert);
-        if(notValidAfter == (time_t)-1) {
-            xmlSecGnuTLSError2("gnutls_x509_crt_get_expiration_time", GNUTLS_E_SUCCESS,
-                NULL,
-                "cert expiration time is invalid: %.lf",
-                difftime(notValidAfter, (time_t)0));
-            return(-1);
-       }
-
-        /* check */
-        if(ts < notValidBefore) {
-            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_YET_VALID, NULL, NULL);
-            return(0);
-        }
-        if(ts > notValidAfter) {
-            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_HAS_EXPIRED, NULL, NULL);
+        } else if(ret == 0) {
+            /* cert not valid yet or expired */
             return(0);
         }
     }
@@ -250,7 +355,7 @@ xmlSecGnuTLSX509CheckTime(const gnutls_x509_crt_t * cert_list,
  * @store:              the pointer to X509 key data store klass.
  * @certs:              the untrusted certificates.
  * @crls:               the crls.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> element processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; element processing context.
  *
  * Verifies @certs list.
  *
@@ -267,7 +372,10 @@ xmlSecGnuTLSX509StoreVerify(xmlSecKeyDataStorePtr store,
     gnutls_x509_crt_t * cert_list = NULL;
     xmlSecSize cert_list_size;
     gnutls_x509_crl_t * crl_list = NULL;
+    gnutls_x509_crl_t crl;
     xmlSecSize crl_list_size;
+    xmlSecSize crl_ctx_list_size;
+    xmlSecSize crl_actual_list_size = 0;
     gnutls_x509_crt_t * ca_list = NULL;
     xmlSecSize ca_list_size;
     time_t verification_time;
@@ -290,31 +398,49 @@ xmlSecGnuTLSX509StoreVerify(xmlSecKeyDataStorePtr store,
     ctx = xmlSecGnuTLSX509StoreGetCtx(store);
     xmlSecAssert2(ctx != NULL, NULL);
 
+    /* gnutls doesn't allow to specify "verification" timestamp so
+     * we have to do it ourselves */
+    verification_time = (keyInfoCtx->certsVerificationTime > 0) ?
+                        keyInfoCtx->certsVerificationTime :
+                        time(0);
+
     /* Prepare */
     cert_list_size = certs_size + xmlSecPtrListGetSize(&(ctx->certsUntrusted));
     if(cert_list_size > 0) {
         cert_list = (gnutls_x509_crt_t *)xmlMalloc(sizeof(gnutls_x509_crt_t) * cert_list_size);
         if(cert_list == NULL) {
             xmlSecMallocError(sizeof(gnutls_x509_crt_t) * cert_list_size,
-                              xmlSecKeyDataStoreGetName(store));
+                xmlSecKeyDataStoreGetName(store));
             goto done;
         }
     }
+
     crl_list_size = xmlSecPtrListGetSize(crls);
-    if(crl_list_size > 0) {
-        crl_list = (gnutls_x509_crl_t *)xmlMalloc(sizeof(gnutls_x509_crl_t) * crl_list_size);
+    crl_ctx_list_size = xmlSecPtrListGetSize(&(ctx->crls));
+    if((crl_list_size + crl_ctx_list_size) > 0) {
+        crl_list = (gnutls_x509_crl_t *)xmlMalloc(sizeof(gnutls_x509_crl_t) * (crl_list_size + crl_ctx_list_size));
         if(crl_list == NULL) {
-            xmlSecMallocError(sizeof(gnutls_x509_crl_t) * crl_list_size,
-                              xmlSecKeyDataStoreGetName(store));
+            xmlSecMallocError(sizeof(gnutls_x509_crl_t) * (crl_list_size + crl_ctx_list_size),
+                xmlSecKeyDataStoreGetName(store));
             goto done;
         }
         for(ii = 0; ii < crl_list_size; ++ii) {
-            crl_list[ii] = xmlSecPtrListGetItem(crls, ii);
-            if(crl_list[ii] == NULL) {
+            crl = xmlSecPtrListGetItem(crls, ii);
+            if(crl == NULL) {
                 xmlSecInternalError("xmlSecPtrListGetItem(crls)",
-                                    xmlSecKeyDataStoreGetName(store));
+                    xmlSecKeyDataStoreGetName(store));
                 goto done;
             }
+            crl_list[crl_actual_list_size++] = crl;
+        }
+        for(ii = 0; ii < crl_ctx_list_size; ++ii) {
+            crl =  xmlSecPtrListGetItem(&(ctx->crls), ii);
+            if(crl == NULL) {
+                xmlSecInternalError("xmlSecPtrListGetItem(crls)",
+                    xmlSecKeyDataStoreGetName(store));
+                goto done;
+            }
+            crl_list[crl_actual_list_size++] = crl;
         }
     }
 
@@ -336,19 +462,22 @@ xmlSecGnuTLSX509StoreVerify(xmlSecKeyDataStorePtr store,
         }
     }
 
-    /* gnutls doesn't allow to specify "verification" timestamp so
-       we have to do it ourselves */
-    verification_time = (keyInfoCtx->certsVerificationTime > 0) ?
-                        keyInfoCtx->certsVerificationTime :
-                        time(0);
-    flags |= GNUTLS_VERIFY_DISABLE_TIME_CHECKS;
 
+    /* gnutls doesn't allow to specify "verification" timestamp so
+     * we have to do it ourselves. Unfortunately it doesn't work
+     * for CRLs yet: https://github.com/lsh123/xmlsec/issues/579
+     */
+    if(keyInfoCtx->certsVerificationTime > 0) {
+        flags |= GNUTLS_VERIFY_DISABLE_TIME_CHECKS;
+    }
+
+    flags |= GNUTLS_VERIFY_ALLOW_UNSORTED_CHAIN;
     if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS) != 0) {
         flags |= GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2;
         flags |= GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5;
 #if GNUTLS_VERSION_NUMBER >= 0x030600
         flags |= GNUTLS_VERIFY_ALLOW_SIGN_WITH_SHA1;
-#endif
+#endif /* GNUTLS_VERSION_NUMBER >= 0x030600 */
     }
 
     /* We are going to build all possible cert chains and try to verify them */
@@ -364,45 +493,36 @@ xmlSecGnuTLSX509StoreVerify(xmlSecKeyDataStorePtr store,
             goto done;
         }
 
-        if (xmlSecGnuTLSX509CertIsSelfSigned(cert) != 1) {
-            /* check if we are the "leaf" node in the certs chain */
-            if (xmlSecGnuTLSX509FindSignedCert(certs, cert) != NULL) {
-                continue;
-            }
-
-            /* build the chain */
-            for (cert2 = cert, cert_list_cur_size = 0;
-                (cert2 != NULL) && (cert_list_cur_size < cert_list_size);
-                ++cert_list_cur_size)
-            {
-                gnutls_x509_crt_t tmp;
-
-                /* store */
-                cert_list[cert_list_cur_size] = cert2;
-
-                /* find next */
-                tmp = xmlSecGnuTLSX509FindSignerCert(certs, cert2);
-                if (tmp == NULL) {
-                    tmp = xmlSecGnuTLSX509FindSignerCert(&(ctx->certsUntrusted), cert2);
-                }
-                cert2 = tmp;
-            }
-        } else if (certs_size == 1) {
-            /* only do self signed cert when it is the only cert */
-            /* chain for self signed cert is easy */
-            cert_list[0] = cert;
-            cert_list_cur_size = 1;
-        } else {
+        /* check if we are the "leaf" node in the certs chain */
+        if(xmlSecGnuTLSX509FindSignedCert(certs, cert) != NULL) {
             continue;
         }
 
+        /* build the chain */
+        for(cert2 = cert, cert_list_cur_size = 0;
+            (cert2 != NULL) && (cert_list_cur_size < cert_list_size);
+            ++cert_list_cur_size)
+        {
+            gnutls_x509_crt_t tmp;
+
+            /* store */
+            cert_list[cert_list_cur_size] = cert2;
+
+            /* find next */
+            tmp = xmlSecGnuTLSX509FindSignerCert(certs, cert2);
+            if(tmp == NULL) {
+                tmp = xmlSecGnuTLSX509FindSignerCert(&(ctx->certsUntrusted), cert2);
+            }
+            cert2 = tmp;
+        }
+
         /* try to verify */
         if((keyInfoCtx->flags & XMLSEC_KEYINFO_FLAGS_X509DATA_DONT_VERIFY_CERTS) == 0) {
             unsigned int cert_list_cur_len, ca_list_len, crl_list_len;
 
             XMLSEC_SAFE_CAST_SIZE_TO_UINT(cert_list_cur_size, cert_list_cur_len, goto done, NULL);
             XMLSEC_SAFE_CAST_SIZE_TO_UINT(ca_list_size, ca_list_len, goto done, NULL);
-            XMLSEC_SAFE_CAST_SIZE_TO_UINT(crl_list_size, crl_list_len, goto done, NULL);
+            XMLSEC_SAFE_CAST_SIZE_TO_UINT(crl_actual_list_size, crl_list_len, goto done, NULL);
 
             err = gnutls_x509_crt_list_verify(
                     cert_list, cert_list_cur_len, /* certs chain */
@@ -426,9 +546,9 @@ xmlSecGnuTLSX509StoreVerify(xmlSecKeyDataStorePtr store,
 
         /* gnutls doesn't allow to specify "verification" timestamp so
            we have to do it ourselves */
-        ret = xmlSecGnuTLSX509CheckTime(cert_list, cert_list_cur_size, verification_time);
+        ret = xmlSecGnuTLSX509CheckCrtsTime(cert_list, cert_list_cur_size, verification_time);
         if(ret != 1) {
-            xmlSecInternalError("xmlSecGnuTLSX509CheckTime", NULL);
+            xmlSecInternalError("xmlSecGnuTLSX509CheckCrtsTime", NULL);
             /* ignore error, don't stop, continue! */
             continue;
         }
@@ -493,6 +613,38 @@ xmlSecGnuTLSX509StoreAdoptCert(xmlSecKeyDataStorePtr store, gnutls_x509_crt_t ce
     return(0);
 }
 
+
+/**
+ * xmlSecGnuTLSX509StoreAdoptCrl:
+ * @store:              the pointer to X509 key data store klass.
+ * @crl:                the pointer to GnuTLS X509 CRL.
+ *
+ * Adds CRL to the store.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int
+xmlSecGnuTLSX509StoreAdoptCrl(xmlSecKeyDataStorePtr store, gnutls_x509_crl_t crl) {
+    xmlSecGnuTLSX509StoreCtxPtr ctx;
+    int ret;
+
+    xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecGnuTLSX509StoreId), -1);
+    xmlSecAssert2(crl != NULL, -1);
+
+    ctx = xmlSecGnuTLSX509StoreGetCtx(store);
+    xmlSecAssert2(ctx != NULL, -1);
+
+   ret = xmlSecPtrListAdd(&(ctx->crls), crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecPtrListAdd(crls)", xmlSecKeyDataStoreGetName(store));
+        return(-1);
+    }
+
+    /* done */
+    return(0);
+}
+
+
 static int
 xmlSecGnuTLSX509StoreInitialize(xmlSecKeyDataStorePtr store) {
     xmlSecGnuTLSX509StoreCtxPtr ctx;
@@ -519,6 +671,13 @@ xmlSecGnuTLSX509StoreInitialize(xmlSecKeyDataStorePtr store) {
         return(-1);
     }
 
+    ret = xmlSecPtrListInitialize(&(ctx->crls), xmlSecGnuTLSX509CrlListId);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecPtrListInitialize(crls)",
+                            xmlSecKeyDataStoreGetName(store));
+        return(-1);
+    }
+
     return(0);
 }
 
@@ -532,6 +691,7 @@ xmlSecGnuTLSX509StoreFinalize(xmlSecKeyDataStorePtr store) {
 
     xmlSecPtrListFinalize(&(ctx->certsTrusted));
     xmlSecPtrListFinalize(&(ctx->certsUntrusted));
+    xmlSecPtrListFinalize(&(ctx->crls));
 
     memset(ctx, 0, sizeof(xmlSecGnuTLSX509StoreCtx));
 }
@@ -544,7 +704,7 @@ xmlSecGnuTLSX509StoreFinalize(xmlSecKeyDataStorePtr store) {
  *****************************************************************************/
 #define XMLSEC_GNUTLS_DN_ATTRS_SIZE             1024
 
-static int
+int
 xmlSecGnuTLSX509DnsEqual(const xmlChar * ll, const xmlChar * rr) {
     xmlSecGnuTLSDnAttr ll_attrs[XMLSEC_GNUTLS_DN_ATTRS_SIZE];
     xmlSecGnuTLSDnAttr rr_attrs[XMLSEC_GNUTLS_DN_ATTRS_SIZE];
@@ -600,7 +760,7 @@ done:
  *
  * Returns 0 if SKI matches, 1 if SKI doesn't match and a negative value if an error occurs.
  */
-static int
+int
 xmlSecGnuTLSX509CertCompareSKI(gnutls_x509_crt_t cert, const xmlSecByte * ski, xmlSecSize skiSize) {
     xmlSecByte* buf = NULL;
     size_t bufSizeT = 0;
@@ -660,86 +820,33 @@ done:
 }
 
 static gnutls_x509_crt_t
-xmlSecGnuTLSX509FindCert(xmlSecPtrListPtr certs, const xmlChar *subjectName,
-                         const xmlChar *issuerName, const xmlChar *issuerSerial,
-                         const xmlSecByte * ski, xmlSecSize skiSize) {
+xmlSecGnuTLSX509FindCert(xmlSecPtrListPtr certs, xmlSecGnuTLSX509FindCertCtxPtr findCertCtx) {
     xmlSecSize ii, sz;
+    int ret;
 
     xmlSecAssert2(certs != NULL, NULL);
+    xmlSecAssert2(findCertCtx != NULL, NULL);
 
     /* todo: this is not the fastest way to search certs */
     sz = xmlSecPtrListGetSize(certs);
     for(ii = 0; (ii < sz); ++ii) {
         gnutls_x509_crt_t cert = xmlSecPtrListGetItem(certs, ii);
         if(cert == NULL) {
-            xmlSecInternalError2("xmlSecPtrListGetItem", NULL,
-                "pos=" XMLSEC_SIZE_FMT, ii);
+            xmlSecInternalError2("xmlSecPtrListGetItem", NULL, "pos=" XMLSEC_SIZE_FMT, ii);
             return(NULL);
         }
 
-        /* check subject name */
-        if(subjectName != NULL) {
-            xmlChar * tmp;
 
-            tmp = xmlSecGnuTLSX509CertGetSubjectDN(cert);
-            if(tmp == NULL) {
-                xmlSecInternalError2("xmlSecGnuTLSX509CertGetSubjectDN", NULL,
-                    "pos=" XMLSEC_SIZE_FMT, ii);
-                return(NULL);
-            }
-
-            if(xmlSecGnuTLSX509DnsEqual(subjectName, tmp) == 1) {
-                xmlFree(tmp);
-                return(cert);
-            }
-            xmlFree(tmp);
-        }
-
-        /* check issuer name + serial */
-        if((issuerName != NULL) && (issuerSerial != NULL)) {
-            xmlChar * tmp1;
-            xmlChar * tmp2;
-
-            tmp1 = xmlSecGnuTLSX509CertGetIssuerDN(cert);
-            if(tmp1 == NULL) {
-                xmlSecInternalError2("xmlSecGnuTLSX509CertGetIssuerDN", NULL,
-                    "pos=" XMLSEC_SIZE_FMT, ii);
-                return(NULL);
-            }
-
-            tmp2 = xmlSecGnuTLSX509CertGetIssuerSerial(cert);
-            if(tmp2 == NULL) {
-                xmlSecInternalError2("xmlSecGnuTLSX509CertGetIssuerSerial", NULL,
-                    "pos=" XMLSEC_SIZE_FMT, ii);
-                xmlFree(tmp1);
-                return(NULL);
-            }
-
-            if((xmlSecGnuTLSX509DnsEqual(issuerName, tmp1) == 1) && xmlStrEqual(issuerSerial, tmp2)) {
-                xmlFree(tmp1);
-                xmlFree(tmp2);
-                return(cert);
-            }
-            xmlFree(tmp1);
-            xmlFree(tmp2);
-        }
-
-        /* check subject ski */
-        if((ski != NULL) && (skiSize > 0)) {
-            int ret;
-
-            ret = xmlSecGnuTLSX509CertCompareSKI(cert, ski, skiSize);
-            if(ret < 0) {
-                xmlSecInternalError2("xmlSecGnuTLSX509CertCompareSKI", NULL,
-                    "pos=" XMLSEC_SIZE_FMT, ii);
-                return(NULL);
-            }
-            if(ret == 0) {
-                return(cert);
-            }
+        /* returns 1 for match, 0 for no match, and a negative value if an error occurs */
+        ret = xmlSecGnuTLSX509FindCertCtxMatch(findCertCtx, cert);
+        if(ret < 0) {
+            xmlSecInternalError2("xmlSecGnuTLSX509FindCertCtxMatch", NULL, "pos=" XMLSEC_SIZE_FMT, ii);
+            return(NULL);
+        } else if(ret == 1) {
+            return(cert);
         }
     }
-
+    /* not found */
     return(NULL);
 }
 
@@ -846,5 +953,3 @@ done:
 }
 
 #endif /* XMLSEC_NO_X509 */
-
-
