@@ -6,17 +6,19 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_KEYSDATA_HELPERS_H__
 #define __XMLSEC_KEYSDATA_HELPERS_H__
 
 
 #ifndef XMLSEC_PRIVATE
-#error "private.h file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
+#error "keysdata_helpers.h file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
 #endif /* XMLSEC_PRIVATE */
 
 #include <xmlsec/keysdata.h>
+#include <xmlsec/keysmngr.h>
+#include <xmlsec/x509.h>
 
 /**************************************************************************
  *
@@ -73,6 +75,175 @@ XMLSEC_EXPORT void              xmlSecKeyDataBinaryValueDebugXmlDump    (xmlSecK
                                                                          FILE* output);
 
 
+#if !defined(XMLSEC_NO_EC)
+
+/* TODO: do we even need pub_x and pub_y? */
+typedef struct _xmlSecKeyValueEc {
+    xmlChar* curve;
+    xmlSecBuffer pubkey;
+    xmlSecBuffer pub_x;
+    xmlSecBuffer pub_y;
+} xmlSecKeyValueEc, *xmlSecKeyValueEcPtr;
+
+/**
+ * xmlSecKeyDataEcRead:
+ * @id:                 the key data data.
+ * @ecValue:            the pointer to input @xmlSecKeyValueEc.
+ *
+ * Creates xmlSecKeyData from @ecValue
+ *
+ * Returns: the poitner to xmlSecKeyData or NULL if an error occurs.
+ */
+typedef xmlSecKeyDataPtr       (*xmlSecKeyDataEcRead)                   (xmlSecKeyDataId id,
+                                                                         xmlSecKeyValueEcPtr ecValue);
+
+/**
+ * xmlSecKeyDataEcWrite:
+ * @id:                 the key data data.
+ * @data:               the pointer to input @xmlSecKeyData.
+ * @c:                  the pointer to input @xmlSecKeyValueEc.
+ *
+ * Writes @xmlSecKeyData to @xmlSecKeyValueEc.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+typedef int                    (*xmlSecKeyDataEcWrite)                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyDataPtr data,
+                                                                         xmlSecKeyValueEcPtr ecValue);
+
+
+XMLSEC_EXPORT int               xmlSecKeyDataEcPublicKeySplitComponents (xmlSecKeyValueEcPtr ecValue);
+XMLSEC_EXPORT int               xmlSecKeyDataEcPublicKeyCombineComponents (xmlSecKeyValueEcPtr ecValue);
+
+
+XMLSEC_EXPORT int               xmlSecKeyDataEcXmlRead                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         xmlSecKeyDataEcRead readFunc);
+XMLSEC_EXPORT int               xmlSecKeyDataEcXmlWrite                 (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         int base64LineSize,
+                                                                         int addLineBreaks,
+                                                                         xmlSecKeyDataEcWrite writeFunc);
+
+#endif /* !defined(XMLSEC_NO_EC) */
+
+#if !defined(XMLSEC_NO_RSA)
+/**************************************************************************
+ *
+ * Helper functions to read/write RSA keys
+ *
+ *************************************************************************/
+typedef struct _xmlSecKeyValueRsa {
+    xmlSecBuffer   modulus;
+    xmlSecBuffer   publicExponent;
+    xmlSecBuffer   privateExponent;
+} xmlSecKeyValueRsa, *xmlSecKeyValueRsaPtr;
+
+/**
+ * xmlSecKeyDataRsaRead:
+ * @id:                 the key data data.
+ * @dsaValue:            the pointer to input @xmlSecKeyValueRsa.
+ *
+ * Creates xmlSecKeyData from @dsaValue
+ *
+ * Returns: the poitner to xmlSecKeyData or NULL if an error occurs.
+ */
+typedef xmlSecKeyDataPtr       (*xmlSecKeyDataRsaRead)                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyValueRsaPtr rsaValue);
+
+/**
+ * xmlSecKeyDataRsaWrite:
+ * @id:                 the key data data.
+ * @data:               the pointer to input @xmlSecKeyData.
+ * @dsaValue:            the pointer to input @xmlSecKeyValueRsa.
+ * @writePrivateKey:    the flag indicating if private key component should be output or not.
+ *
+ * Writes @xmlSecKeyData to @xmlSecKeyValueRsa.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+typedef int                    (*xmlSecKeyDataRsaWrite)                 (xmlSecKeyDataId id,
+                                                                         xmlSecKeyDataPtr data,
+                                                                         xmlSecKeyValueRsaPtr rsaValue,
+                                                                         int writePrivateKey);
+
+
+XMLSEC_EXPORT int               xmlSecKeyDataRsaXmlRead                 (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         xmlSecKeyDataRsaRead readFunc);
+XMLSEC_EXPORT int               xmlSecKeyDataRsaXmlWrite                (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         int base64LineSize,
+                                                                         int addLineBreaks,
+                                                                         xmlSecKeyDataRsaWrite writeFunc);
+#endif /* !defined(XMLSEC_NO_RSA) */
+
+#if !defined(XMLSEC_NO_DH)
+/**************************************************************************
+ *
+ * Helper functions to read/write DH keys
+ *
+ *************************************************************************/
+typedef struct _xmlSecKeyValueDh {
+    xmlSecBuffer p;
+    xmlSecBuffer q;
+    xmlSecBuffer generator;
+    xmlSecBuffer public;
+    xmlSecBuffer seed;
+    xmlSecBuffer pgenCounter;
+} xmlSecKeyValueDh, *xmlSecKeyValueDhPtr;
+
+/**
+ * xmlSecKeyDataDhRead:
+ * @id:                 the key data data.
+ * @dhValue:            the pointer to input @xmlSecKeyValueDh.
+ *
+ * Creates xmlSecKeyData from @dhValue
+ *
+ * Returns: the poitner to xmlSecKeyData or NULL if an error occurs.
+ */
+typedef xmlSecKeyDataPtr       (*xmlSecKeyDataDhRead)                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyValueDhPtr dhValue);
+
+/**
+ * xmlSecKeyDataDhWrite:
+ * @id:                 the key data data.
+ * @data:               the pointer to input @xmlSecKeyData.
+ * @dhValue:            the pointer to input @xmlSecKeyValueDh.
+ * @writePrivateKey:    the flag indicating if private key component should be output or not.
+ *
+ * Writes @xmlSecKeyData to @xmlSecKeyValueDh.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+typedef int                    (*xmlSecKeyDataDhWrite)                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyDataPtr data,
+                                                                         xmlSecKeyValueDhPtr dhValue,
+                                                                         int writePrivateKey);
+
+XMLSEC_EXPORT int               xmlSecKeyDataDhXmlRead                  (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         xmlSecKeyDataDhRead readFunc);
+XMLSEC_EXPORT int               xmlSecKeyDataDhXmlWrite                 (xmlSecKeyDataId id,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlNodePtr node,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
+                                                                         int base64LineSize,
+                                                                         int addLineBreaks,
+                                                                         xmlSecKeyDataDhWrite writeFunc);
+#endif /* !defined(XMLSEC_NO_DH) */
+
+
 #if !defined(XMLSEC_NO_DSA)
 /**************************************************************************
  *
@@ -129,110 +300,48 @@ XMLSEC_EXPORT int               xmlSecKeyDataDsaXmlWrite                (xmlSecK
                                                                          xmlSecKeyDataDsaWrite writeFunc);
 #endif /* !defined(XMLSEC_NO_DSA) */
 
-#if !defined(XMLSEC_NO_RSA)
-/**************************************************************************
- *
- * Helper functions to read/write RSA keys
- *
- *************************************************************************/
-typedef struct _xmlSecKeyValueRsa {
-    xmlSecBuffer   modulus;
-    xmlSecBuffer   publicExponent;
-    xmlSecBuffer   privateExponent;
-} xmlSecKeyValueRsa, *xmlSecKeyValueRsaPtr;
-
-/**
- * xmlSecKeyDataRsaRead:
- * @id:                 the key data data.
- * @dsaValue:            the pointer to input @xmlSecKeyValueRsa.
- *
- * Creates xmlSecKeyData from @dsaValue
- *
- * Returns: the poitner to xmlSecKeyData or NULL if an error occurs.
- */
-typedef xmlSecKeyDataPtr       (*xmlSecKeyDataRsaRead)                  (xmlSecKeyDataId id,
-                                                                         xmlSecKeyValueRsaPtr rsaValue);
-
-/**
- * xmlSecKeyDataRsaWrite:
- * @id:                 the key data data.
- * @data:               the pointer to input @xmlSecKeyData.
- * @dsaValue:            the pointer to input @xmlSecKeyValueRsa.
- * @writePrivateKey:    the flag indicating if private key component should be output or not.
- *
- * Writes @xmlSecKeyData to @xmlSecKeyValueRsa.
- *
- * Returns: 0 on success or a negative value if an error occurs.
- */
-typedef int                    (*xmlSecKeyDataRsaWrite)                 (xmlSecKeyDataId id,
-                                                                         xmlSecKeyDataPtr data,
-                                                                         xmlSecKeyValueRsaPtr rsaValue,
-                                                                         int writePrivateKey);
-
-
-XMLSEC_EXPORT int               xmlSecKeyDataRsaXmlRead                 (xmlSecKeyDataId id,
-                                                                         xmlSecKeyPtr key,
-                                                                         xmlNodePtr node,
-                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
-                                                                         xmlSecKeyDataRsaRead readFunc);
-XMLSEC_EXPORT int               xmlSecKeyDataRsaXmlWrite                (xmlSecKeyDataId id,
-                                                                         xmlSecKeyPtr key,
-                                                                         xmlNodePtr node,
-                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx,
-                                                                         int base64LineSize,
-                                                                         int addLineBreaks,
-                                                                         xmlSecKeyDataRsaWrite writeFunc);
-#endif /* !defined(XMLSEC_NO_RSA) */
-
 #if !defined(XMLSEC_NO_X509)
 /**************************************************************************
  *
  * Helper functions to read/write X509 Keys
  *
  *************************************************************************/
-typedef struct _xmlSecKeyValueX509 {
-    xmlSecBuffer cert;
-    xmlSecBuffer crl;
-    xmlSecBuffer ski;
-    xmlChar* subject;
-    xmlChar* issuerName;
-    xmlChar* issuerSerial;
-} xmlSecKeyValueX509, *xmlSecKeyValueX509Ptr;
+
 
 /**
  * xmlSecKeyDataX509Read:
- * @data:               the pointer to result @xmlSecKeyData.
- * @x509Value:          the pointer to input @xmlSecKeyValueX509.
+ * @data:               the pointer to X509 key data
+ * @x509Value:          the pointer to input @xmlSecKeyX509DataValue.
  * @keysMngr:           the pointer to @xmlSecKeysMngr.
  * @flags:              the flags for certs processing.
  *
- * Creates xmlSecKeyData from @dsaValue
  *
- * Returns: the poitner to xmlSecKeyData or NULL if an error occurs.
+ * Returns: 0 on success and a negative value otherwise.
  */
 typedef int                    (*xmlSecKeyDataX509Read)                 (xmlSecKeyDataPtr data,
-                                                                         xmlSecKeyValueX509Ptr x509Value,
+                                                                         xmlSecKeyX509DataValuePtr x509Value,
                                                                          xmlSecKeysMngrPtr keysMngr,
                                                                          unsigned int flags);
 
 /**
  * xmlSecKeyDataX509Write:
  * @data:               the pointer to result @xmlSecKeyData.
- * @x509Value:          the pointer to result @xmlSecKeyValueX509.
+ * @x509Value:          the pointer to result @xmlSecKeyX509DataValue.
  * @content:            the bitmask of what should be output to @x509Value.
  * @context:            the writer function context.
  *
  * If available, writes the next X509 object (cert or crl) into @x509Value.
  *
- * Returns: 0 on success, 1 if no more certs/crls are available, or a negative'
+ * Returns: 1 on success, 0 if no more certs/crls are available, or a negative
  * value if an error occurs.
  */
 typedef int                    (*xmlSecKeyDataX509Write)                (xmlSecKeyDataPtr data,
-                                                                         xmlSecKeyValueX509Ptr x509Value,
+                                                                         xmlSecKeyX509DataValuePtr x509Value,
                                                                          int content,
                                                                          void* context);
 
-XMLSEC_EXPORT int               xmlSecKeyDataX509XmlRead                (xmlSecKeyDataPtr data,
+XMLSEC_EXPORT int               xmlSecKeyDataX509XmlRead                (xmlSecKeyPtr key,
+                                                                         xmlSecKeyDataPtr data,
                                                                          xmlNodePtr node,
                                                                          xmlSecKeyInfoCtxPtr keyInfoCtx,
                                                                          xmlSecKeyDataX509Read readFunc);
