@@ -28,17 +28,19 @@
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/base64.h>
-#include <xmlsec/keys.h>
-#include <xmlsec/keyinfo.h>
-#include <xmlsec/transforms.h>
-#include <xmlsec/errors.h>
 #include <xmlsec/bn.h>
+#include <xmlsec/errors.h>
+#include <xmlsec/keyinfo.h>
+#include <xmlsec/keys.h>
+#include <xmlsec/transforms.h>
+#include <xmlsec/xmltree.h>
 
-#include <xmlsec/mscng/crypto.h>
 #include <xmlsec/mscng/certkeys.h>
+#include <xmlsec/mscng/crypto.h>
 
 #include "../cast_helpers.h"
 #include "../keysdata_helpers.h"
+#include "private.h"
 
 typedef struct _xmlSecMSCngKeyDataCtx xmlSecMSCngKeyDataCtx,
                                       *xmlSecMSCngKeyDataCtxPtr;
@@ -52,15 +54,13 @@ struct _xmlSecMSCngKeyDataCtx {
 XMLSEC_KEY_DATA_DECLARE(MSCngKeyData, xmlSecMSCngKeyDataCtx)
 #define xmlSecMSCngKeyDataSize XMLSEC_KEY_DATA_SIZE(MSCngKeyData)
 
-static xmlSecSize xmlSecMSCngKeyDataGetSize(xmlSecKeyDataPtr data);
-
 static int
 xmlSecMSCngKeyDataCertGetPubkey(PCCERT_CONTEXT cert, BCRYPT_KEY_HANDLE* key) {
     xmlSecAssert2(cert != NULL, -1);
     xmlSecAssert2(key != NULL, -1);
 
     if(!CryptImportPublicKeyInfoEx2(X509_ASN_ENCODING,
-            &(cert->pCertInfo->SubjectPublicKeyInfo),
+            &cert->pCertInfo->SubjectPublicKeyInfo,
             0,
             NULL,
             key)) {
@@ -164,6 +164,53 @@ xmlSecMSCngKeyDataAdoptKey(xmlSecKeyDataPtr data, BCRYPT_KEY_HANDLE hPubKey) {
     return(0);
 }
 
+static xmlSecKeyDataPtr
+xmlSecMSCngKeyDataFromAlgorithm(LPSTR pszObjId) {
+    xmlSecKeyDataPtr data = NULL;
+
+    xmlSecAssert2(pszObjId != NULL, NULL);
+
+#ifndef XMLSEC_NO_DSA
+    if (!strcmp(pszObjId, szOID_X957_DSA)) {
+        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataDsaId);
+        if (data == NULL) {
+            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataDsaId)", NULL);
+            return(NULL);
+        }
+    }
+#endif /* XMLSEC_NO_DSA */
+
+#ifndef XMLSEC_NO_RSA
+    if (!strcmp(pszObjId, szOID_RSA_RSA)) {
+        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataRsaId);
+        if (data == NULL) {
+            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataRsaId)", NULL);
+            return(NULL);
+        }
+    }
+#endif /* XMLSEC_NO_RSA */
+
+#ifndef XMLSEC_NO_EC
+    if (!strcmp(pszObjId, szOID_ECC_PUBLIC_KEY)) {
+        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataEcId);
+        if (data == NULL) {
+            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataEcId)", NULL);
+            return(NULL);
+        }
+    }
+#endif /* XMLSEC_NO_EC */
+
+    if (data == NULL) {
+        xmlSecInvalidStringTypeError("Algorithm",
+            pszObjId,
+            "unsupported keytype",
+            NULL);
+        return(NULL);
+    }
+
+    return(data);
+}
+
 /**
  * xmlSecMSCngCertAdopt:
  * @pCert:              the pointer to cert.
@@ -182,41 +229,9 @@ xmlSecMSCngCertAdopt(PCCERT_CONTEXT pCert, xmlSecKeyDataType type) {
     xmlSecAssert2(pCert->pCertInfo != NULL, NULL);
     xmlSecAssert2(pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId != NULL, NULL);
 
-#ifndef XMLSEC_NO_DSA
-    if(!strcmp(pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId, szOID_X957_DSA)) {
-        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataDsaId);
-        if(data == NULL) {
-            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataDsaId)", NULL);
-            return(NULL);
-        }
-    }
-#endif /* XMLSEC_NO_DSA */
-
-#ifndef XMLSEC_NO_RSA
-    if(!strcmp(pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId, szOID_RSA_RSA)) {
-        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataRsaId);
-        if(data == NULL) {
-            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataRsaId)", NULL);
-            return(NULL);
-        }
-    }
-#endif /* XMLSEC_NO_RSA */
-
-#ifndef XMLSEC_NO_ECDSA
-    if(!strcmp(pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId, szOID_ECC_PUBLIC_KEY)) {
-        data = xmlSecKeyDataCreate(xmlSecMSCngKeyDataEcdsaId);
-        if(data == NULL) {
-            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataEcdsaId)", NULL);
-            return(NULL);
-        }
-    }
-#endif /* XMLSEC_NO_ECDSA */
-
-    if(data == NULL) {
-        xmlSecInvalidStringTypeError("PCCERT_CONTEXT key type",
-            pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId,
-            "unsupported keytype",
-            NULL);
+    data = xmlSecMSCngKeyDataFromAlgorithm(pCert->pCertInfo->SubjectPublicKeyInfo.Algorithm.pszObjId);
+    if (data == NULL) {
+        xmlSecInternalError("xmlSecMSCngKeyDataAdoptCert", NULL);
         return(NULL);
     }
 
@@ -411,11 +426,27 @@ xmlSecMSCngKeyDataDuplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
                 pszAlgId = BCRYPT_DSA_ALGORITHM;
                 break;
 #endif
+
 #ifndef XMLSEC_NO_RSA
             case BCRYPT_RSAPUBLIC_MAGIC:
                 pszAlgId = BCRYPT_RSA_ALGORITHM;
                 break;
 #endif
+
+#ifndef XMLSEC_NO_EC
+            case BCRYPT_ECDH_PUBLIC_P256_MAGIC:
+            case BCRYPT_ECDSA_PUBLIC_P256_MAGIC:
+                pszAlgId = BCRYPT_ECDSA_P256_ALGORITHM;
+                break;
+            case BCRYPT_ECDH_PUBLIC_P384_MAGIC:
+            case BCRYPT_ECDSA_PUBLIC_P384_MAGIC:
+                pszAlgId = BCRYPT_ECDSA_P384_ALGORITHM;
+                break;
+            case BCRYPT_ECDH_PUBLIC_P521_MAGIC:
+            case BCRYPT_ECDSA_PUBLIC_P521_MAGIC:
+                pszAlgId = BCRYPT_ECDSA_P521_ALGORITHM;
+                break;
+#endif
             default:
                 xmlSecNotImplementedError(NULL);
                 xmlFree(pbBlob);
@@ -457,6 +488,305 @@ xmlSecMSCngKeyDataDuplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
     return(0);
 }
 
+
+
+/**************************************************************************
+ *
+ * <dsig11:DEREncodedKeyValue /> processing
+ *
+ *************************************************************************/
+static int                      xmlSecMSCngKeyDataDEREncodedKeyValueXmlRead(xmlSecKeyDataId id,
+    xmlSecKeyPtr key,
+    xmlNodePtr node,
+    xmlSecKeyInfoCtxPtr keyInfoCtx);
+static int                      xmlSecMSCngKeyDataDEREncodedKeyValueXmlWrite(xmlSecKeyDataId id,
+    xmlSecKeyPtr key,
+    xmlNodePtr node,
+    xmlSecKeyInfoCtxPtr keyInfoCtx);
+
+
+
+static xmlSecKeyDataKlass xmlSecMSCngKeyDataDEREncodedKeyValueKlass = {
+    sizeof(xmlSecKeyDataKlass),
+    sizeof(xmlSecKeyData),
+
+    /* data */
+    xmlSecNameDEREncodedKeyValue,
+    xmlSecKeyDataUsageKeyInfoNode | xmlSecKeyDataUsageRetrievalMethodNodeXml, /* xmlSecKeyDataUsage usage; */
+    NULL,                                       /* const xmlChar* href; */
+    xmlSecNodeDEREncodedKeyValue,               /* const xmlChar* dataNodeName; */
+    xmlSecDSig11Ns,                             /* const xmlChar* dataNodeNs; */
+
+    /* constructors/destructor */
+    NULL,                                       /* xmlSecKeyDataInitializeMethod initialize; */
+    NULL,                                       /* xmlSecKeyDataDuplicateMethod duplicate; */
+    NULL,                                       /* xmlSecKeyDataFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecKeyDataGenerateMethod generate; */
+
+    /* get info */
+    NULL,                                       /* xmlSecKeyDataGetTypeMethod getType; */
+    NULL,                                       /* xmlSecKeyDataGetSizeMethod getSize; */
+    NULL,                                       /* xmlSecKeyDataGetIdentifier getIdentifier; */
+
+    /* read/write */
+    xmlSecMSCngKeyDataDEREncodedKeyValueXmlRead,     /* xmlSecKeyDataXmlReadMethod xmlRead; */
+    xmlSecMSCngKeyDataDEREncodedKeyValueXmlWrite,    /* xmlSecKeyDataXmlWriteMethod xmlWrite; */
+    NULL,                                       /* xmlSecKeyDataBinReadMethod binRead; */
+    NULL,                                       /* xmlSecKeyDataBinWriteMethod binWrite; */
+
+    /* debug */
+    NULL,                                       /* xmlSecKeyDataDebugDumpMethod debugDump; */
+    NULL,                                       /* xmlSecKeyDataDebugDumpMethod debugXmlDump; */
+
+    /* reserved for the future */
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngKeyDataDEREncodedKeyValueGetKlass:
+ * The public key algorithm and value are DER-encoded in accordance with the value that would be used
+ * in the Subject Public Key Info field of an X.509 certificate, per section 4.1.2.7 of [RFC5280].
+ * The DER-encoded value is then base64-encoded.
+ *
+ * https://www.w3.org/TR/xmldsig-core1/#sec-DEREncodedKeyValue
+ *
+ *      <!-- targetNamespace="http://www.w3.org/2009/xmldsig11#" -->
+ *      <element name="DEREncodedKeyValue" type="dsig11:DEREncodedKeyValueType" />
+ *      <complexType name="DEREncodedKeyValueType">
+ *          <simpleContent>
+ *              <extension base="base64Binary">
+ *                  <attribute name="Id" type="ID" use="optional"/>
+ *              </extension>
+ *          </simpleContent>
+ *      </complexType>
+ *
+ * Returns: the &lt;dsig11:DEREncodedKeyValue/&gt;element processing key data klass.
+ */
+xmlSecKeyDataId
+xmlSecMSCngKeyDataDEREncodedKeyValueGetKlass(void) {
+    return(&xmlSecMSCngKeyDataDEREncodedKeyValueKlass);
+}
+
+static int
+xmlSecMSCngKeyDataDEREncodedKeyValueXmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key, xmlNodePtr node, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecBuffer buffer;
+    const xmlSecByte* data;
+    xmlSecSize dataSize;
+    DWORD dataLen;
+    LPVOID keyInfo = NULL;
+    DWORD keyInfoLen = 0;
+    BCRYPT_KEY_HANDLE hPubkey = 0;
+    xmlSecKeyDataPtr keyData = NULL;
+    xmlNodePtr cur;
+    BOOL status;
+    int res = -1;
+    int ret;
+
+    xmlSecAssert2(id == xmlSecMSCngKeyDataDEREncodedKeyValueId, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(node != NULL, -1);
+    xmlSecAssert2(node->doc != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+    xmlSecAssert2(keyInfoCtx->mode == xmlSecKeyInfoModeRead, -1);
+
+    ret = xmlSecBufferInitialize(&buffer, 256);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBufferInitialize", xmlSecKeyDataKlassGetName(id));
+        return(-1);
+    }
+
+    /* no children are expected */
+    cur = xmlSecGetNextElementNode(node->children);
+    if (cur != NULL) {
+        xmlSecUnexpectedNodeError(cur, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* read base64 node content */
+    ret = xmlSecBufferBase64NodeContentRead(&buffer, node);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBufferBase64NodeContentRead", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    data = xmlSecBufferGetData(&buffer);
+    dataSize = xmlSecBufferGetSize(&buffer);
+    if ((data == NULL) || (dataSize <= 0)) {
+        /* this is not an error if we are reading a doc to be encrypted or signed */
+        res = 0;
+        goto done;
+    }
+
+    /* read pubkey */
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(dataSize, dataLen, goto done, xmlSecKeyDataKlassGetName(id));
+    status = CryptDecodeObjectEx(
+        X509_ASN_ENCODING,
+        X509_PUBLIC_KEY_INFO,
+        data,
+        dataLen,
+        CRYPT_DECODE_ALLOC_FLAG,
+        NULL,
+        &keyInfo,
+        &keyInfoLen
+    );
+    if ((status != TRUE) || (keyInfo == NULL) || (keyInfoLen <= 0)) {
+        xmlSecMSCngNtError("CryptDecodeObjectEx", xmlSecKeyDataKlassGetName(id), STATUS_SUCCESS);
+        goto done;
+    }
+
+    status = CryptImportPublicKeyInfoEx2(
+        X509_ASN_ENCODING,
+        (PCERT_PUBLIC_KEY_INFO)keyInfo,
+        0,
+        NULL,
+        &hPubkey
+    );
+    if ((status != TRUE) || (hPubkey == 0)) {
+        xmlSecMSCngNtError("CryptDecodeObjectEx", xmlSecKeyDataKlassGetName(id), STATUS_SUCCESS);
+        goto done;
+    }
+
+    /* add to key */
+    keyData = xmlSecMSCngKeyDataFromAlgorithm(((PCERT_PUBLIC_KEY_INFO)keyInfo)->Algorithm.pszObjId);
+    if (keyData == NULL) {
+        xmlSecInternalError("xmlSecMSCngKeyDataFromAlgorithm", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    ret = xmlSecMSCngKeyDataAdoptKey(keyData, hPubkey);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngKeyDataAdoptKey", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    hPubkey = 0; /* owned by key data now */
+
+    ret = xmlSecKeySetValue(key, keyData);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecKeySetValue", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    keyData = NULL; /* owned by key now */
+
+    /* success! */
+    res = 0;
+
+done:
+
+    if (keyData != NULL) {
+        xmlSecKeyDataDestroy(keyData);
+    }
+    if (hPubkey != 0) {
+        BCryptDestroyKey(hPubkey);
+    }
+    if (keyInfo != NULL) {
+        LocalFree(keyInfo);
+    }
+    xmlSecBufferFinalize(&buffer);
+    return(res);
+}
+
+static int
+xmlSecMSCngKeyDataDEREncodedKeyValueXmlWrite(xmlSecKeyDataId id, xmlSecKeyPtr key, xmlNodePtr node, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecKeyDataPtr keyData;
+    BCRYPT_KEY_HANDLE hPubkey;
+    PUCHAR pInfo = NULL;
+    DWORD cbInfo = 0;
+    LPVOID keyDer = NULL;
+    DWORD keyDerLen = 0;
+    xmlChar* content = NULL;
+    BOOL status;
+    int res = -1;
+
+    xmlSecAssert2(id == xmlSecMSCngKeyDataDEREncodedKeyValueId, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(node != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+    xmlSecAssert2(keyInfoCtx->mode == xmlSecKeyInfoModeWrite, -1);
+
+    /* get pubkey */
+    keyData = xmlSecKeyGetValue(key);
+    if (keyData == NULL) {
+        xmlSecInternalError("xmlSecKeyGetValue", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    hPubkey = xmlSecMSCngKeyDataGetPubKey(keyData);
+    if (hPubkey == 0) {
+        xmlSecInternalError("xmlSecMSCngKeyDataGetPubKey", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* encode it */
+    status = CryptExportPublicKeyInfoFromBCryptKeyHandle(
+        hPubkey,
+        X509_ASN_ENCODING,
+        NULL,
+        0,
+        NULL,
+        NULL,
+        &cbInfo
+    );
+    if ((status != TRUE) || (cbInfo <= 0)) {
+        xmlSecMSCngNtError("CryptExportPublicKeyInfoFromBCryptKeyHandle", xmlSecKeyDataKlassGetName(id), STATUS_SUCCESS);
+        goto done;
+    }
+    pInfo = (PUCHAR)xmlMalloc(cbInfo);
+    if (pInfo == NULL) {
+        xmlSecMallocError(cbInfo, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    status = CryptExportPublicKeyInfoFromBCryptKeyHandle(
+        hPubkey,
+        X509_ASN_ENCODING,
+        NULL,
+        0,
+        NULL,
+        (PCERT_PUBLIC_KEY_INFO )pInfo,
+        &cbInfo
+    );
+    if ((status != TRUE) || (cbInfo <= 0)) {
+        xmlSecMSCngNtError("CryptExportPublicKeyInfoFromBCryptKeyHandle", xmlSecKeyDataKlassGetName(id), STATUS_SUCCESS);
+        goto done;
+    }
+
+    status = CryptEncodeObjectEx(
+        X509_ASN_ENCODING,
+        X509_PUBLIC_KEY_INFO,
+        pInfo,
+        CRYPT_ENCODE_ALLOC_FLAG,
+        NULL,
+        &keyDer,
+        &keyDerLen
+    );
+    if ((status != TRUE) || (keyDer == NULL) || (keyDerLen <= 0)) {
+        xmlSecMSCngNtError("CryptEncodeObjectEx", xmlSecKeyDataKlassGetName(id), STATUS_SUCCESS);
+        goto done;
+    }
+
+    /* write to XML */
+    content = xmlSecBase64Encode(keyDer, keyDerLen, xmlSecBase64GetDefaultLineSize());
+    if (content == NULL) {
+        xmlSecInternalError("xmlSecBase64Encode", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    xmlNodeAddContent(node, content);
+
+    /* success */
+    res = 0;
+
+done:
+    if (pInfo != NULL) {
+        xmlFree(pInfo);
+    }
+    if (keyDer != NULL) {
+        LocalFree(keyDer);
+    }
+    if (content != NULL) {
+        xmlFree(content);
+    }
+    return(res);
+}
+
+
 #ifndef XMLSEC_NO_DSA
 
 #define XMLSEC_MSCNG_DSA_MAX_Q_SIZE     (20U)
@@ -873,7 +1203,7 @@ static xmlSecKeyDataKlass xmlSecMSCngKeyDataDsaKlass = {
 
     /* data */
     xmlSecNameDSAKeyValue,
-    xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefDSAKeyValue,                      /* const xmlChar* href; */
     xmlSecNodeDSAKeyValue,                      /* const xmlChar* dataNodeName; */
@@ -1190,7 +1520,7 @@ xmlSecMSCngKeyDataRsaGetType(xmlSecKeyDataPtr data) {
     return(xmlSecKeyDataTypePublic);
 }
 
-static xmlSecSize
+xmlSecSize
 xmlSecMSCngKeyDataGetSize(xmlSecKeyDataPtr data) {
     NTSTATUS status;
     xmlSecMSCngKeyDataCtxPtr ctx;
@@ -1342,7 +1672,7 @@ static xmlSecKeyDataKlass xmlSecMSCngKeyDataRsaKlass = {
 
     /* data */
     xmlSecNameRSAKeyValue,
-    xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefRSAKeyValue,                      /* const xmlChar* href; */
     xmlSecNodeRSAKeyValue,                      /* const xmlChar* dataNodeName; */
@@ -1387,20 +1717,20 @@ xmlSecMSCngKeyDataRsaGetKlass(void) {
 }
 #endif /* XMLSEC_NO_RSA */
 
-#ifndef XMLSEC_NO_ECDSA
+#ifndef XMLSEC_NO_EC
 static int
-xmlSecMSCngKeyDataEcdsaDuplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
-    xmlSecAssert2(xmlSecKeyDataCheckId(dst, xmlSecMSCngKeyDataEcdsaId), -1);
-    xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecMSCngKeyDataEcdsaId), -1);
+xmlSecMSCngKeyDataEcDuplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(dst, xmlSecMSCngKeyDataEcId), -1);
+    xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecMSCngKeyDataEcId), -1);
 
     return(xmlSecMSCngKeyDataDuplicate(dst, src));
 }
 
 static xmlSecKeyDataType
-xmlSecMSCngKeyDataEcdsaGetType(xmlSecKeyDataPtr data) {
+xmlSecMSCngKeyDataEcGetType(xmlSecKeyDataPtr data) {
     xmlSecMSCngKeyDataCtxPtr ctx;
 
-    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcdsaId), xmlSecKeyDataTypeUnknown);
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcId), xmlSecKeyDataTypeUnknown);
 
     ctx = xmlSecMSCngKeyDataGetCtx(data);
     xmlSecAssert2(ctx != NULL, xmlSecKeyDataTypeUnknown);
@@ -1413,77 +1743,392 @@ xmlSecMSCngKeyDataEcdsaGetType(xmlSecKeyDataPtr data) {
 }
 
 static xmlSecSize
-xmlSecMSCngKeyDataEcdsaGetSize(xmlSecKeyDataPtr data) {
-    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcdsaId), 0);
+xmlSecMSCngKeyDataEcGetSize(xmlSecKeyDataPtr data) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcId), 0);
 
     return(xmlSecMSCngKeyDataGetSize(data));
 }
 
 
 static void
-xmlSecMSCngKeyDataEcdsaDebugDump(xmlSecKeyDataPtr data, FILE* output) {
-    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcdsaId));
+xmlSecMSCngKeyDataEcDebugDump(xmlSecKeyDataPtr data, FILE* output) {
+    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcId));
     xmlSecAssert(output != NULL);
 
     fprintf(output, "=== rsa key: size = " XMLSEC_SIZE_FMT "\n",
-        xmlSecMSCngKeyDataEcdsaGetSize(data));
+        xmlSecMSCngKeyDataEcGetSize(data));
 }
 
-static void xmlSecMSCngKeyDataEcdsaDebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
-    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcdsaId));
+static void xmlSecMSCngKeyDataEcDebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
+    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcId));
     xmlSecAssert(output != NULL);
 
-    fprintf(output, "<ECDSAKeyValue size=\"" XMLSEC_SIZE_FMT "\" />\n",
-        xmlSecMSCngKeyDataEcdsaGetSize(data));
+    fprintf(output, "<ECKeyValue size=\"" XMLSEC_SIZE_FMT "\" />\n",
+        xmlSecMSCngKeyDataEcGetSize(data));
 }
 
-static xmlSecKeyDataKlass xmlSecMSCngKeyDataEcdsaKlass = {
+typedef struct _xmlSecMSCngKeyDataEccCurveNameAndMagic {
+    ULONG magic;
+    LPCWSTR blobType;
+    xmlChar oid[128];
+} xmlSecMSCngKeyDataEccCurveNameAndMagic;
+
+/* https://learn.microsoft.com/en-us/windows/win32/api/bcrypt/ns-bcrypt-bcrypt_ecckey_blob */
+static const xmlSecMSCngKeyDataEccCurveNameAndMagic g_xmlSecMSCngKeyDataEccCurveNameAndMagic[] = {
+    { BCRYPT_ECDSA_PUBLIC_P256_MAGIC, BCRYPT_ECDSA_P256_ALGORITHM, "1.2.840.10045.3.1.7" }, /* prime256v1 */
+    { BCRYPT_ECDSA_PUBLIC_P384_MAGIC, BCRYPT_ECDSA_P384_ALGORITHM, "1.3.132.0.34" }, /* secp384r1 */
+    { BCRYPT_ECDSA_PUBLIC_P521_MAGIC, BCRYPT_ECDSA_P521_ALGORITHM, "1.3.132.0.35" }  /* secp521r1 */
+};
+
+
+static const xmlChar*
+xmlSecOpenSSLKeyDataEcGetOidFromMagic(ULONG magic) {
+    xmlSecSize size = sizeof(g_xmlSecMSCngKeyDataEccCurveNameAndMagic) / sizeof(g_xmlSecMSCngKeyDataEccCurveNameAndMagic[0]);
+
+    xmlSecAssert2(magic != 0, NULL);
+
+    for (xmlSecSize ii = 0; ii < size; ++ii) {
+        if (magic == g_xmlSecMSCngKeyDataEccCurveNameAndMagic[ii].magic) {
+            return(g_xmlSecMSCngKeyDataEccCurveNameAndMagic[ii].oid);
+        }
+    }
+    return(NULL);
+}
+
+static LPCWSTR
+xmlSecOpenSSLKeyDataEcGetTypeAndMagicFromOid(const xmlChar* oid, ULONG * magic) {
+    xmlSecSize size = sizeof(g_xmlSecMSCngKeyDataEccCurveNameAndMagic) / sizeof(g_xmlSecMSCngKeyDataEccCurveNameAndMagic[0]);
+
+    xmlSecAssert2(oid != NULL, 0);
+    xmlSecAssert2(magic != NULL, 0);
+
+    for (xmlSecSize ii = 0; ii < size; ++ii) {
+        if (xmlStrcmp(oid, g_xmlSecMSCngKeyDataEccCurveNameAndMagic[ii].oid) == 0) {
+            (*magic) = g_xmlSecMSCngKeyDataEccCurveNameAndMagic[ii].magic;
+            return(g_xmlSecMSCngKeyDataEccCurveNameAndMagic[ii].blobType);
+        }
+    }
+    return(0);
+}
+
+static xmlSecKeyDataPtr
+xmlSecMSCngKeyDataEcRead(xmlSecKeyDataId id, xmlSecKeyValueEcPtr ecValue) {
+    xmlSecKeyDataPtr data = NULL;
+    xmlSecKeyDataPtr res = NULL;
+    xmlSecBuffer blob;
+    int blobInitialized = 0;
+    xmlSecByte* blobData;
+    xmlSecByte* pubkeyData;
+    xmlSecSize pubkeySize;
+    xmlSecSize offset, blobSize;
+    DWORD dwBlobSize;
+    BCRYPT_ECCKEY_BLOB* eckey;
+    LPCWSTR blobType;
+    BCRYPT_KEY_HANDLE hKey = NULL;
+    NTSTATUS status;
+    BCRYPT_ALG_HANDLE hAlg = NULL;
+    int ret;
+
+    xmlSecAssert2(id == xmlSecMSCngKeyDataEcId, NULL);
+    xmlSecAssert2(ecValue != NULL, NULL);
+    xmlSecAssert2(ecValue->curve != NULL, NULL);
+
+    /* first byte in ecValue->pubkey is the magical byte, we don't need it */
+    pubkeyData = xmlSecBufferGetData(&(ecValue->pubkey));
+    pubkeySize = xmlSecBufferGetSize(&(ecValue->pubkey));
+    xmlSecAssert2(pubkeyData != NULL, NULL);
+    xmlSecAssert2(pubkeySize > 1, NULL);
+    pubkeyData += 1;
+    pubkeySize -= 1;
+
+    /* turn the read data into a public key blob, as documented at
+     * https://learn.microsoft.com/en-us/windows/win32/api/bcrypt/ns-bcrypt-bcrypt_ecckey_blob>
+     *
+     * dont reverse blobs as both the XML and CNG works with big-endian
+     *
+     */
+    offset = sizeof(BCRYPT_ECCKEY_BLOB);
+    blobSize = offset + pubkeySize;
+
+    ret = xmlSecBufferInitialize(&blob, blobSize);
+    if (ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetSize", xmlSecKeyDataKlassGetName(id),
+            "size=" XMLSEC_SIZE_FMT, blobSize);
+        goto done;
+    }
+    blobInitialized = 1;
+
+    ret = xmlSecBufferSetSize(&blob, blobSize);
+    if (ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetSize", xmlSecKeyDataKlassGetName(id),
+            "size=" XMLSEC_SIZE_FMT, blobSize);
+        goto done;
+    }
+    memset(xmlSecBufferGetData(&blob), 0, blobSize); // ensure all padding with 0s work
+
+    blobData = xmlSecBufferGetData(&blob);
+    eckey = (BCRYPT_ECCKEY_BLOB*)blobData;
+    blobType = xmlSecOpenSSLKeyDataEcGetTypeAndMagicFromOid(ecValue->curve, &(eckey->dwMagic));
+    if ((blobType == NULL) || (eckey->dwMagic == 0)) {
+        xmlSecInternalError2("xmlSecOpenSSLKeyDataEcGetOidFromNid", xmlSecKeyDataKlassGetName(id),
+            "curve=%s", xmlSecErrorsSafeString(ecValue->curve));
+        goto done;
+    }
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(pubkeySize / 2, eckey->cbKey, goto done, xmlSecKeyDataKlassGetName(id));
+
+    /* pubkey */
+    memcpy(blobData + offset, pubkeyData, pubkeySize);
+
+    /* import the key blob */
+    status = BCryptOpenAlgorithmProvider(
+        &hAlg,
+        blobType,
+        NULL,
+        0);
+    if (status != STATUS_SUCCESS) {
+        xmlSecMSCngNtError("BCryptOpenAlgorithmProvider", xmlSecKeyDataKlassGetName(id), status);
+        goto done;
+    }
+
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(blobSize, dwBlobSize, goto done, xmlSecKeyDataKlassGetName(id));
+    status = BCryptImportKeyPair(
+        hAlg,
+        NULL,
+        BCRYPT_ECCPUBLIC_BLOB,
+        &hKey,
+        blobData,
+        dwBlobSize,
+        0);
+    if (status != STATUS_SUCCESS) {
+        xmlSecMSCngNtError("BCryptImportKeyPair", xmlSecKeyDataKlassGetName(id), status);
+        goto done;
+    }
+
+    data = xmlSecKeyDataCreate(id);
+    if (data == NULL) {
+        xmlSecInternalError("xmlSecKeyDataCreate", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    ret = xmlSecMSCngKeyDataAdoptKey(data, hKey);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecMSCngKeyDataAdoptKey", xmlSecKeyDataGetName(data));
+        goto done;
+    }
+    hKey = 0; /* now owned by data */
+
+    /* success */
+    res = data;
+    data = NULL;
+
+done:
+    if (data != NULL) {
+        xmlSecKeyDataDestroy(data);
+    }
+    if (hAlg != 0) {
+        BCryptCloseAlgorithmProvider(hAlg, 0);
+    }
+    if (hKey != 0) {
+        BCryptDestroyKey(hKey);
+    }
+    if (blobInitialized != 0) {
+        xmlSecBufferFinalize(&blob);
+    }
+    return(res);
+}
+
+static int
+xmlSecMSCngKeyDataEcWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data, xmlSecKeyValueEcPtr ecValue)
+{
+    xmlSecMSCngKeyDataCtxPtr ctx;
+    NTSTATUS status;
+    xmlSecBuffer buf;
+    int bufInitialized = 0;
+    xmlSecByte* bufData;
+    DWORD bufLen = 0;
+    BCRYPT_ECCKEY_BLOB* eckey;
+    const xmlChar* curve;
+    int ret;
+    int res = -1;
+
+    xmlSecAssert2(id == xmlSecMSCngKeyDataEcId, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecMSCngKeyDataEcId), -1);
+    xmlSecAssert2(ecValue != NULL, -1);
+
+    ctx = xmlSecMSCngKeyDataGetCtx(data);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->pubkey, -1);
+
+    /* turn ctx->pubkey into eckey */
+    status = BCryptExportKey(ctx->pubkey,
+        NULL,
+        BCRYPT_ECCPUBLIC_BLOB,
+        NULL,
+        0,
+        &bufLen,
+        0);
+    if ((status != STATUS_SUCCESS) || (bufLen <= 0)) {
+        xmlSecMSCngNtError2("BCryptExportKey", xmlSecKeyDataKlassGetName(id),
+            status, "bufLen=%lu", bufLen);
+        goto done;
+    }
+
+    ret = xmlSecBufferInitialize(&buf, bufLen);
+    if (ret < 0) {
+        xmlSecInternalError2("xmlSecBufferInitialize", xmlSecKeyDataKlassGetName(id),
+            "size=%lu", bufLen);
+        goto done;
+    }
+    bufInitialized = 1;
+
+    bufData = xmlSecBufferGetData(&buf);
+    xmlSecAssert2(bufData != NULL, -1);
+
+    status = BCryptExportKey(ctx->pubkey,
+        NULL,
+        BCRYPT_ECCPUBLIC_BLOB,
+        bufData,
+        bufLen,
+        &bufLen,
+        0);
+    if ((status != STATUS_SUCCESS) || (bufLen <= 0)) {
+        xmlSecMSCngNtError2("BCryptExportKey", xmlSecKeyDataKlassGetName(id),
+            status, "bufLen=%lu", bufLen);
+        goto done;
+    }
+
+    /* check BCRYPT_ECCKEY_BLOB */
+    if (bufLen < sizeof(BCRYPT_ECCKEY_BLOB)) {
+        xmlSecMSCngNtError2("BCRYPT_ECCKEY_BLOB", xmlSecKeyDataKlassGetName(id),
+            STATUS_SUCCESS, "dwBlobLen=%lu", bufLen);
+        goto done;
+    }
+    eckey = (BCRYPT_ECCKEY_BLOB*)bufData;
+    bufData += sizeof(BCRYPT_ECCKEY_BLOB);
+    bufLen  -= (DWORD)sizeof(BCRYPT_ECCKEY_BLOB);
+    if (bufLen != 2 * eckey->cbKey) {
+        xmlSecMSCngNtError3("BCRYPT_ECCKEY_BLOB", xmlSecKeyDataKlassGetName(id),
+            STATUS_SUCCESS, "bufLen=%lu, eckey->cbKey=%lu", bufLen, eckey->cbKey);
+        goto done;
+    }
+    /* curve */
+    curve = xmlSecOpenSSLKeyDataEcGetOidFromMagic(eckey->dwMagic);
+    if (curve == NULL) {
+        xmlSecInternalError2("xmlSecOpenSSLKeyDataEcGetOidFromMagic", xmlSecKeyDataKlassGetName(id),
+            "magic=%lu", eckey->dwMagic);
+        goto done;
+    }
+    ecValue->curve = xmlStrdup(curve);
+    if (ecValue->curve == NULL) {
+        xmlSecStrdupError(curve, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* public key is prefixed with magic 0x04 */
+    xmlSecByte magic[] = { 0x04 };
+    ret = xmlSecBufferSetData(&(ecValue->pubkey), magic, sizeof(magic));
+    if (ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetData(pubkey)", xmlSecKeyDataKlassGetName(id),
+            "magic size=%d", (int)sizeof(magic));
+        goto done;
+    }
+    ret = xmlSecBufferAppend(&(ecValue->pubkey), bufData, bufLen);
+    if (ret < 0) {
+        xmlSecInternalError2("xmlSecBufferAppend(pubkey)", xmlSecKeyDataKlassGetName(id),
+            "bufLen=%lu", bufLen);
+        goto done;
+    }
+
+    /* dont reverse blobs as both the XML and CNG works with big-endian */
+
+    /* success */
+    res = 0;
+
+done:
+    if (bufInitialized != 0) {
+        xmlSecBufferFinalize(&buf);
+    }
+    return(res);
+}
+
+
+static int
+xmlSecMSCngKeyDataEcXmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key,
+    xmlNodePtr node, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecAssert2(id == xmlSecMSCngKeyDataEcId, -1);
+    return(xmlSecKeyDataEcXmlRead(id, key, node, keyInfoCtx,
+        xmlSecMSCngKeyDataEcRead));
+}
+
+static int
+xmlSecMSCngKeyDataEcXmlWrite(xmlSecKeyDataId id, xmlSecKeyPtr key,
+    xmlNodePtr node, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecAssert2(id == xmlSecMSCngKeyDataEcId, -1);
+    return(xmlSecKeyDataEcXmlWrite(id, key, node, keyInfoCtx,
+        xmlSecBase64GetDefaultLineSize(), 1, /* add line breaks */
+        xmlSecMSCngKeyDataEcWrite));
+}
+
+static xmlSecKeyDataKlass xmlSecMSCngKeyDataEcKlass = {
     sizeof(xmlSecKeyDataKlass),
     xmlSecMSCngKeyDataSize,
 
     /* data */
-    xmlSecNameECDSAKeyValue,
-    xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecNameECKeyValue,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
-    xmlSecHrefECDSAKeyValue,                    /* const xmlChar* href; */
-    xmlSecNodeECDSAKeyValue,                    /* const xmlChar* dataNodeName; */
-    xmlSecDSigNs,                               /* const xmlChar* dataNodeNs; */
+    xmlSecHrefECKeyValue,                       /* const xmlChar* href; */
+    xmlSecNodeECKeyValue,                       /* const xmlChar* dataNodeName; */
+    xmlSecDSig11Ns,                             /* const xmlChar* dataNodeNs; */
 
     /* constructors/destructor */
     xmlSecMSCngKeyDataInitialize,               /* xmlSecKeyDataInitializeMethod initialize; */
-    xmlSecMSCngKeyDataEcdsaDuplicate,           /* xmlSecKeyDataDuplicateMethod duplicate; */
+    xmlSecMSCngKeyDataEcDuplicate,              /* xmlSecKeyDataDuplicateMethod duplicate; */
     xmlSecMSCngKeyDataFinalize,                 /* xmlSecKeyDataFinalizeMethod finalize; */
     NULL,                                       /* xmlSecKeyDataGenerateMethod generate; */
 
     /* get info */
-    xmlSecMSCngKeyDataEcdsaGetType,             /* xmlSecKeyDataGetTypeMethod getType; */
-    xmlSecMSCngKeyDataEcdsaGetSize,             /* xmlSecKeyDataGetSizeMethod getSize; */
+    xmlSecMSCngKeyDataEcGetType,                /* xmlSecKeyDataGetTypeMethod getType; */
+    xmlSecMSCngKeyDataEcGetSize,                /* xmlSecKeyDataGetSizeMethod getSize; */
     NULL,                                       /* xmlSecKeyDataGetIdentifier getIdentifier; */
 
     /* read/write */
-    NULL,                                       /* xmlSecKeyDataXmlReadMethod xmlRead; */
-    NULL,                                       /* xmlSecKeyDataXmlWriteMethod xmlWrite; */
+    xmlSecMSCngKeyDataEcXmlRead,                /* xmlSecKeyDataXmlReadMethod xmlRead; */
+    xmlSecMSCngKeyDataEcXmlWrite,               /* xmlSecKeyDataXmlWriteMethod xmlWrite; */
     NULL,                                       /* xmlSecKeyDataBinReadMethod binRead; */
     NULL,                                       /* xmlSecKeyDataBinWriteMethod binWrite; */
 
     /* debug */
-    xmlSecMSCngKeyDataEcdsaDebugDump,           /* xmlSecKeyDataDebugDumpMethod debugDump; */
-    xmlSecMSCngKeyDataEcdsaDebugXmlDump,        /* xmlSecKeyDataDebugDumpMethod debugXmlDump; */
+    xmlSecMSCngKeyDataEcDebugDump,              /* xmlSecKeyDataDebugDumpMethod debugDump; */
+    xmlSecMSCngKeyDataEcDebugXmlDump,           /* xmlSecKeyDataDebugDumpMethod debugXmlDump; */
 
     /* reserved for the future */
     NULL,                                       /* void* reserved0; */
     NULL,                                       /* void* reserved1; */
 };
 
+/**
+ * xmlSecMSCngKeyDataEcGetKlass:
+ *
+ * The MSCng EC CertKey data klass.
+ *
+ * Returns: pointer to MSCng EC key data klass.
+ */
+xmlSecKeyDataId
+xmlSecMSCngKeyDataEcGetKlass(void) {
+    return(&xmlSecMSCngKeyDataEcKlass);
+}
+
 /**
  * xmlSecMSCngKeyDataEcdsaGetKlass:
  *
- * The MSCng ECDSA CertKey data klass.
+ * Deprecated. The MSCng EC CertKey data klass.
  *
- * Returns: pointer to MSCng ECDSA key data klass.
+ * Returns: pointer to MSCng EC key data klass.
  */
 xmlSecKeyDataId
 xmlSecMSCngKeyDataEcdsaGetKlass(void) {
-    return(&xmlSecMSCngKeyDataEcdsaKlass);
+    return(xmlSecMSCngKeyDataEcGetKlass());
 }
-#endif /* XMLSEC_NO_ECDSA */
+#endif /* XMLSEC_NO_EC */
