@@ -1,6 +1,7 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Signatures implementation for MSCng.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
@@ -8,10 +9,7 @@
  * Copyright (C) 2018 Miklos Vajna. All Rights Reserved.
  */
 /**
- * SECTION:signatures
- * @Short_description: Signatures implementation for Microsoft Cryptography API: Next Generation (CNG).
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -33,6 +31,20 @@
 #include <xmlsec/mscng/certkeys.h>
 
 #include "../cast_helpers.h"
+#include "private.h"
+
+ /*************************************************************************
+  *
+  * DSA EVP
+  *
+  * https://www.w3.org/TR/xmldsig-core1/#sec-DSA
+  * The output of the DSA algorithm consists of a pair of integers usually referred by the pair (r, s).
+  * DSA-SHA1: Integer to octet-stream conversion must be done according to the I2OSP operation defined
+  *           in the RFC 3447 [PKCS1] specification with a l parameter equal to 20
+  * DSA-SHA256: The pairs (2048, 256) and (3072, 256) correspond to the algorithm DSAwithSHA256
+  ************************************************************************/
+#define XMLSEC_MSCNG_SIGNATURE_DSA_SHA1_HALF_LEN              20
+#define XMLSEC_MSCNG_SIGNATURE_DSA_SHA256_HALF_LEN            (256 / 8)
 
 /**************************************************************************
  *
@@ -44,12 +56,15 @@ typedef struct _xmlSecMSCngSignatureCtx      xmlSecMSCngSignatureCtx,
 struct _xmlSecMSCngSignatureCtx {
     xmlSecKeyDataPtr    data;
     xmlSecKeyDataId     keyId;
+    DWORD               signatureHalfSize;
     LPCWSTR pszHashAlgId;
     DWORD cbHash;
     PBYTE pbHash;
     BCRYPT_ALG_HANDLE hHashAlg;
     PBYTE pbHashObject;
     BCRYPT_HASH_HANDLE hHash;
+    ULONG dwInfoFlags;
+    ULONG dwRsaPssSaltSize;
 };
 
 /******************************************************************************
@@ -120,9 +135,33 @@ static int xmlSecMSCngSignatureCheckId(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
+#ifndef XMLSEC_NO_SHA1
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha1Id)) {
+        return(1);
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA256
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha256Id)) {
+        return(1);
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha384Id)) {
+        return(1);
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha512Id)) {
+        return(1);
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+
 #endif /* XMLSEC_NO_RSA */
 
-#ifndef XMLSEC_NO_ECDSA
+#ifndef XMLSEC_NO_EC
 
 #ifndef XMLSEC_NO_SHA1
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha1Id)) {
@@ -148,7 +187,7 @@ static int xmlSecMSCngSignatureCheckId(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
-#endif /* XMLSEC_NO_ECDSA */
+#endif /* XMLSEC_NO_EC */
 
     /* not found */
     return(0);
@@ -171,6 +210,7 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformDsaSha1Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA1_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataDsaId;
+        ctx->signatureHalfSize = XMLSEC_MSCNG_SIGNATURE_DSA_SHA1_HALF_LEN;
     } else
 #endif /* XMLSEC_NO_SHA1 */
 
@@ -182,6 +222,7 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaMd5Id)) {
         ctx->pszHashAlgId = BCRYPT_MD5_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PKCS1;
     } else
 #endif /* XMLSEC_NO_MD5 */
 
@@ -189,6 +230,7 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaSha1Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA1_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PKCS1;
     } else
 #endif /* XMLSEC_NO_SHA1 */
 
@@ -196,6 +238,7 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaSha256Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA256_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PKCS1;
     } else
 #endif /* XMLSEC_NO_SHA256 */
 
@@ -203,6 +246,7 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaSha384Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA384_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PKCS1;
     } else
 #endif /* XMLSEC_NO_SHA384 */
 
@@ -210,42 +254,79 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaSha512Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA512_ALGORITHM;
         ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PKCS1;
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+
+#ifndef XMLSEC_NO_SHA1
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha1Id)) {
+        ctx->pszHashAlgId = BCRYPT_SHA1_ALGORITHM;
+        ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PSS;
+        ctx->dwRsaPssSaltSize = 20; /* The default salt length is the length of the hash function. */
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA256
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha256Id)) {
+        ctx->pszHashAlgId = BCRYPT_SHA256_ALGORITHM;
+        ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PSS;
+        ctx->dwRsaPssSaltSize = 32; /* The default salt length is the length of the hash function. */
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha384Id)) {
+        ctx->pszHashAlgId = BCRYPT_SHA384_ALGORITHM;
+        ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PSS;
+        ctx->dwRsaPssSaltSize = 48; /* The default salt length is the length of the hash function. */
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if (xmlSecTransformCheckId(transform, xmlSecMSCngTransformRsaPssSha512Id)) {
+        ctx->pszHashAlgId = BCRYPT_SHA512_ALGORITHM;
+        ctx->keyId = xmlSecMSCngKeyDataRsaId;
+        ctx->dwInfoFlags = BCRYPT_PAD_PSS;
+        ctx->dwRsaPssSaltSize = 64; /* The default salt length is the length of the hash function. */
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
 #endif /* XMLSEC_NO_RSA */
 
-#ifndef XMLSEC_NO_ECDSA
+#ifndef XMLSEC_NO_EC
 
 #ifndef XMLSEC_NO_SHA1
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha1Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA1_ALGORITHM;
-        ctx->keyId = xmlSecMSCngKeyDataEcdsaId;
+        ctx->keyId = xmlSecMSCngKeyDataEcId;
     } else
 #endif /* XMLSEC_NO_SHA1 */
 
 #ifndef XMLSEC_NO_SHA256
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha256Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA256_ALGORITHM;
-        ctx->keyId = xmlSecMSCngKeyDataEcdsaId;
+        ctx->keyId = xmlSecMSCngKeyDataEcId;
     } else
 #endif /* XMLSEC_NO_SHA256 */
 
 #ifndef XMLSEC_NO_SHA384
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha384Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA384_ALGORITHM;
-        ctx->keyId = xmlSecMSCngKeyDataEcdsaId;
+        ctx->keyId = xmlSecMSCngKeyDataEcId;
     } else
 #endif /* XMLSEC_NO_SHA384 */
 
 #ifndef XMLSEC_NO_SHA512
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha512Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA512_ALGORITHM;
-        ctx->keyId = xmlSecMSCngKeyDataEcdsaId;
+        ctx->keyId = xmlSecMSCngKeyDataEcId;
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
-#endif /* XMLSEC_NO_ECDSA */
+#endif /* XMLSEC_NO_EC */
 
     /* not found */
     {
@@ -347,16 +428,93 @@ static int xmlSecMSCngSignatureSetKeyReq(xmlSecTransformPtr transform,  xmlSecKe
     return(0);
 }
 
-static int xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
-                                      const xmlSecByte* data,
-                                      xmlSecSize dataSize,
-                                      xmlSecTransformCtxPtr transformCtx) {
+/*
+* https://www.w3.org/TR/xmldsig-core1/#sec-ECDSA
+* 
+* The output of the ECDSA algorithm consists of a pair of integers usually
+* referred by the pair(r, s).The signature value consists of the base64
+* encoding of the concatenation of two octet - streams that respectively result
+* from the octet - encoding of the values r and s in that order.Integer to
+* octet - stream conversion must be done according to the I2OSP operation defined
+* in the RFC 3447[PKCS1] specification with the l parameter equal to the size of
+* the base point order of the curve in bytes(e.g. 32 for the P - 256 curve and 66
+* for the P - 521 curve).
+*/
+static int
+xmlSecMSCngSignatureFixBrokenJava(xmlSecMSCngSignatureCtxPtr ctx,
+    const xmlSecByte* data, xmlSecSize dataSize,
+    const xmlSecByte** out, xmlSecSize* outSize
+) {
+    xmlSecSize halfSize;
+    xmlSecSize keySize;
+    xmlSecByte* res;
+    xmlSecSize offset;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->data != NULL, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(out != NULL, -1);
+    xmlSecAssert2(outSize != NULL, -1);
+
+    if (ctx->keyId == xmlSecMSCngKeyDataDsaId) {
+        halfSize = ctx->signatureHalfSize;
+    } else if (ctx->keyId == xmlSecMSCngKeyDataEcId) {
+        keySize = xmlSecMSCngKeyDataGetSize(ctx->data);
+        if (keySize <= 0) {
+            xmlSecInternalError("xmlSecMSCngKeyDataGetSize", NULL);
+            return(-1);
+        }
+        halfSize = (keySize + 7) / 8;
+    } else {
+        /* nothing to do: this only applies to DSA and ECDSA signatures */
+        return(0);
+    }
+
+    /* check the size */
+    if (2 * halfSize == dataSize) {
+        return(0);
+    }
+    else if ((dataSize > 2 * halfSize) || (dataSize % 2 != 0)) {
+        xmlSecInternalError3("xmlSecOpenSSLEvpSignatureEcdsaHalfLen", NULL,
+            "expectedSignLen=" XMLSEC_SIZE_FMT "; actualSignLen=" XMLSEC_SIZE_FMT, 2 * halfSize, dataSize);
+        return(-1);
+    }
+
+    /* let's fix it! */
+    res = (xmlSecByte*)xmlMalloc(2 * halfSize);
+    if (res == NULL) {
+        xmlSecMallocError(2 * halfSize, NULL);
+        return(-1);
+    }
+    memset(res, 0, 2 * halfSize);
+
+    /* add zeros at the beggining of both r and s */
+    offset = (2 * halfSize - dataSize) / 2;
+    memcpy(res + offset, data, dataSize / 2);
+    memcpy(res + halfSize + offset, data + dataSize / 2, dataSize / 2);
+
+    /* success */
+    (*out) = res;
+    (*outSize) = 2 * halfSize;
+    return(0);
+}
+
+static int
+xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
+    const xmlSecByte* data, xmlSecSize dataSize,
+    xmlSecTransformCtxPtr transformCtx
+) {
     xmlSecMSCngSignatureCtxPtr ctx;
     BCRYPT_KEY_HANDLE pubkey;
     NTSTATUS status;
-    BCRYPT_PKCS1_PADDING_INFO info;
-    BCRYPT_PKCS1_PADDING_INFO* pInfo = NULL;
-    DWORD dwDataSize, infoFlags = 0;
+    BCRYPT_PKCS1_PADDING_INFO pkcs1PaddingInfo;
+    BCRYPT_PSS_PADDING_INFO pssPadingInfo;
+    VOID* pPaddingInfo = NULL;
+    xmlSecByte* fixedData = NULL;
+    xmlSecSize fixedDataSize = 0;
+    DWORD dwDataSize;
+    int ret;
+    int res = -1;
 
     xmlSecAssert2(xmlSecMSCngSignatureCheckId(transform), -1);
     xmlSecAssert2(transform->operation == xmlSecTransformOperationVerify, -1);
@@ -372,42 +530,65 @@ static int xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
     pubkey = xmlSecMSCngKeyDataGetPubKey(ctx->data);
     if(pubkey == 0) {
         xmlSecInternalError("xmlSecMSCngKeyDataGetPubKey", xmlSecTransformGetName(transform));
-        return(-1);
+        goto done;
     }
 
-    if(ctx->keyId == xmlSecMSCngKeyDataRsaId) {
-        /* RSA needs explicit padding, otherwise STATUS_INVALID_PARAMETER is
-         * returned */
-        info.pszAlgId = ctx->pszHashAlgId;
-        pInfo = &info;
-        infoFlags = BCRYPT_PAD_PKCS1;
+    /* RSA needs explicit padding, otherwise STATUS_INVALID_PARAMETER is
+     * returned */
+    if(ctx->dwInfoFlags == BCRYPT_PAD_PKCS1) {
+        pkcs1PaddingInfo.pszAlgId = ctx->pszHashAlgId;
+        pPaddingInfo = &pkcs1PaddingInfo;
+    } else if (ctx->dwInfoFlags == BCRYPT_PAD_PSS) {
+        pssPadingInfo.pszAlgId = ctx->pszHashAlgId;
+        pssPadingInfo.cbSalt = ctx->dwRsaPssSaltSize;
+        pPaddingInfo = &pssPadingInfo;
+    } else {
+        /* we expect the DSA/ECDSA r and s to be the same size and either have fixed size (DSA) or match
+         * the size of the key (ECDSA); however some implementations (e.g. Java) cut leading zeros:
+         * https://github.com/lsh123/xmlsec/issues/228 */
+        ret = xmlSecMSCngSignatureFixBrokenJava(ctx, data, dataSize, (const xmlSecByte**)&fixedData, &fixedDataSize);
+        if (ret < 0) {
+            xmlSecInternalError("xmlSecMSCngSignatureFixBrokenJava", xmlSecTransformGetName(transform));
+            goto done;
+        }
+        if ((fixedData != NULL) && (fixedDataSize > 0)) {
+            data = fixedData;
+            dataSize = fixedDataSize;
+        }
     }
 
-    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(dataSize, dwDataSize, return(-1), xmlSecTransformGetName(transform));
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(dataSize, dwDataSize, goto done, xmlSecTransformGetName(transform));
     status = BCryptVerifySignature(
         pubkey,
-        pInfo,
+        pPaddingInfo,
         ctx->pbHash,
         ctx->cbHash,
-        (PBYTE)data,
+        (PBYTE)((fixedData != NULL) ? fixedData : data),
         dwDataSize,
-        infoFlags);
+        ctx->dwInfoFlags);
     if(status != STATUS_SUCCESS) {
         if(status == STATUS_INVALID_SIGNATURE) {
             xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
                 xmlSecTransformGetName(transform),
                 "BCryptVerifySignature: the signature was not verified");
             transform->status = xmlSecTransformStatusFail;
-            return(-1);
+            goto done;
         } else {
             xmlSecMSCngNtError("BCryptVerifySignature",
                 xmlSecTransformGetName(transform), status);
-            return(-1);
+            goto done;
         }
     }
 
+    /* success */
     transform->status = xmlSecTransformStatusOk;
-    return(0);
+    res = 0;
+
+done:
+    if (fixedData != NULL) {
+        xmlFree(fixedData);
+    }
+    return(res);
 }
 
 static int
@@ -418,9 +599,9 @@ xmlSecMSCngSignatureExecute(xmlSecTransformPtr transform, int last, xmlSecTransf
     NTSTATUS status;
     DWORD cbData = 0;
     DWORD cbHashObject = 0;
-    BCRYPT_PKCS1_PADDING_INFO info;
-    BCRYPT_PKCS1_PADDING_INFO* pInfo = NULL;
-    DWORD infoFlags = 0;
+    BCRYPT_PKCS1_PADDING_INFO pkcs1PaddingInfo;
+    BCRYPT_PSS_PADDING_INFO pssPadingInfo;
+    VOID* pPaddingInfo = NULL;
     int ret;
 
     xmlSecAssert2(xmlSecMSCngSignatureCheckId(transform), -1);
@@ -583,21 +764,27 @@ xmlSecMSCngSignatureExecute(xmlSecTransformPtr transform, int last, xmlSecTransf
                     return(-1);
                 }
 
+                /* RSA needs explicit padding, otherwise STATUS_INVALID_PARAMETER is
+                * returned */
+                if(ctx->dwInfoFlags == BCRYPT_PAD_PKCS1) {
+                    pkcs1PaddingInfo.pszAlgId = ctx->pszHashAlgId;
+                    pPaddingInfo = &pkcs1PaddingInfo;
+                } else if (ctx->dwInfoFlags == BCRYPT_PAD_PSS) {
+                    pssPadingInfo.pszAlgId = ctx->pszHashAlgId;
+                    pssPadingInfo.cbSalt = ctx->dwRsaPssSaltSize;
+                    pPaddingInfo = &pssPadingInfo;
+                }
+
                 /* sign the hash */
-        if(ctx->keyId == xmlSecMSCngKeyDataRsaId) {
-                    info.pszAlgId = ctx->pszHashAlgId;
-                    pInfo = &info;
-                    infoFlags = BCRYPT_PAD_PKCS1;
-        }
                 status = NCryptSignHash(
                     privkey,
-                    pInfo,
+                    pPaddingInfo,
                     ctx->pbHash,
                     ctx->cbHash,
                     (PBYTE)xmlSecBufferGetData(&transform->outBuf),
                     cbSignature,
                     &cbSignature,
-                    infoFlags);
+                    ctx->dwInfoFlags);
                 if(status != STATUS_SUCCESS) {
                     xmlSecMSCngNtError("NCryptSignHash",
                         xmlSecTransformGetName(transform), status);
@@ -766,7 +953,7 @@ xmlSecMSCngTransformRsaSha1GetKlass(void) {
 #ifndef XMLSEC_NO_SHA256
 /****************************************************************************
  *
- * RSA-SHA256 signature transform
+ * RSA-SHA2-256 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngRsaSha256Klass = {
@@ -799,9 +986,9 @@ static xmlSecTransformKlass xmlSecMSCngRsaSha256Klass = {
 /**
  * xmlSecMSCngTransformRsaSha256GetKlass:
  *
- * The RSA-SHA256 signature transform klass.
+ * The RSA-SHA2-256 signature transform klass.
  *
- * Returns: RSA-SHA256 signature transform klass.
+ * Returns: RSA-SHA2-256 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformRsaSha256GetKlass(void) {
@@ -812,7 +999,7 @@ xmlSecMSCngTransformRsaSha256GetKlass(void) {
 #ifndef XMLSEC_NO_SHA384
 /****************************************************************************
  *
- * RSA-SHA384 signature transform
+ * RSA-SHA2-384 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngRsaSha384Klass = {
@@ -845,9 +1032,9 @@ static xmlSecTransformKlass xmlSecMSCngRsaSha384Klass = {
 /**
  * xmlSecMSCngTransformRsaSha384GetKlass:
  *
- * The RSA-SHA384 signature transform klass.
+ * The RSA-SHA2-384 signature transform klass.
  *
- * Returns: RSA-SHA384 signature transform klass.
+ * Returns: RSA-SHA2-384 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformRsaSha384GetKlass(void) {
@@ -858,7 +1045,7 @@ xmlSecMSCngTransformRsaSha384GetKlass(void) {
 #ifndef XMLSEC_NO_SHA512
 /****************************************************************************
  *
- * RSA-SHA512 signature transform
+ * RSA-SHA2-512 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngRsaSha512Klass = {
@@ -891,9 +1078,9 @@ static xmlSecTransformKlass xmlSecMSCngRsaSha512Klass = {
 /**
  * xmlSecMSCngTransformRsaSha512GetKlass:
  *
- * The RSA-SHA512 signature transform klass.
+ * The RSA-SHA2-512 signature transform klass.
  *
- * Returns: RSA-SHA512 signature transform klass.
+ * Returns: RSA-SHA2-512 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformRsaSha512GetKlass(void) {
@@ -901,9 +1088,194 @@ xmlSecMSCngTransformRsaSha512GetKlass(void) {
 }
 #endif /* XMLSEC_NO_SHA512 */
 
+
+#ifndef XMLSEC_NO_SHA1
+/****************************************************************************
+ *
+ * RSA-PSS-SHA1 signature transform
+ *
+ ***************************************************************************/
+static xmlSecTransformKlass xmlSecMSCngRsaPssSha1Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),              /* xmlSecSize klassSize */
+    xmlSecMSCngSignatureSize,                  /* xmlSecSize objSize */
+
+    xmlSecNameRsaPssSha1,                      /* const xmlChar* name; */
+    xmlSecHrefRsaPssSha1,                      /* const xmlChar* href; */
+    xmlSecTransformUsageSignatureMethod,       /* xmlSecTransformUsage usage; */
+
+    xmlSecMSCngSignatureInitialize,            /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngSignatureFinalize,              /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                      /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                      /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngSignatureSetKeyReq,             /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    xmlSecMSCngSignatureSetKey,                /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecMSCngSignatureVerify,                /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,         /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,             /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,              /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                      /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                      /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngSignatureExecute,               /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                      /* void* reserved0; */
+    NULL,                                      /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformRsaPssSha1GetKlass:
+ *
+ * The RSA-PSS-SHA1 signature transform klass.
+ *
+ * Returns: RSA-PSS-SHA1 signature transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformRsaPssSha1GetKlass(void) {
+    return(&xmlSecMSCngRsaPssSha1Klass);
+}
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA256
+/****************************************************************************
+ *
+ * RSA-PSS-SHA2-256 signature transform
+ *
+ ***************************************************************************/
+static xmlSecTransformKlass xmlSecMSCngRsaPssSha256Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),              /* xmlSecSize klassSize */
+    xmlSecMSCngSignatureSize,                  /* xmlSecSize objSize */
+
+    xmlSecNameRsaPssSha256,                    /* const xmlChar* name; */
+    xmlSecHrefRsaPssSha256,                    /* const xmlChar* href; */
+    xmlSecTransformUsageSignatureMethod,       /* xmlSecTransformUsage usage; */
+
+    xmlSecMSCngSignatureInitialize,            /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngSignatureFinalize,              /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                      /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                      /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngSignatureSetKeyReq,             /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    xmlSecMSCngSignatureSetKey,                /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecMSCngSignatureVerify,                /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,         /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,             /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,              /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                      /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                      /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngSignatureExecute,               /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                      /* void* reserved0; */
+    NULL,                                      /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformRsaPssSha256GetKlass:
+ *
+ * The RSA-PSS-SHA2-256 signature transform klass.
+ *
+ * Returns: RSA-PSS-SHA2-256 signature transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformRsaPssSha256GetKlass(void) {
+    return(&xmlSecMSCngRsaPssSha256Klass);
+}
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+/****************************************************************************
+ *
+ * RSA-PSS-SHA2-384 signature transform
+ *
+ ***************************************************************************/
+static xmlSecTransformKlass xmlSecMSCngRsaPssSha384Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),              /* xmlSecSize klassSize */
+    xmlSecMSCngSignatureSize,                  /* xmlSecSize objSize */
+
+    xmlSecNameRsaPssSha384,                    /* const xmlChar* name; */
+    xmlSecHrefRsaPssSha384,                    /* const xmlChar* href; */
+    xmlSecTransformUsageSignatureMethod,       /* xmlSecTransformUsage usage; */
+
+    xmlSecMSCngSignatureInitialize,            /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngSignatureFinalize,              /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                      /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                      /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngSignatureSetKeyReq,             /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    xmlSecMSCngSignatureSetKey,                /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecMSCngSignatureVerify,                /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,         /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,             /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,              /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                      /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                      /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngSignatureExecute,               /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                      /* void* reserved0; */
+    NULL,                                      /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformRsaPssSha384GetKlass:
+ *
+ * The RSA-PSS-SHA2-384 signature transform klass.
+ *
+ * Returns: RSA-PSS-SHA2-384 signature transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformRsaPssSha384GetKlass(void) {
+    return(&xmlSecMSCngRsaPssSha384Klass);
+}
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+/****************************************************************************
+ *
+ * RSA-PSS-SHA2-512 signature transform
+ *
+ ***************************************************************************/
+static xmlSecTransformKlass xmlSecMSCngRsaPssSha512Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),              /* xmlSecSize klassSize */
+    xmlSecMSCngSignatureSize,                  /* xmlSecSize objSize */
+
+    xmlSecNameRsaPssSha512,                    /* const xmlChar* name; */
+    xmlSecHrefRsaPssSha512,                    /* const xmlChar* href; */
+    xmlSecTransformUsageSignatureMethod,       /* xmlSecTransformUsage usage; */
+
+    xmlSecMSCngSignatureInitialize,            /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngSignatureFinalize,              /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                      /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                      /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngSignatureSetKeyReq,             /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    xmlSecMSCngSignatureSetKey,                /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecMSCngSignatureVerify,                /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,         /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,             /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,              /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                      /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                      /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngSignatureExecute,               /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                      /* void* reserved0; */
+    NULL,                                      /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformRsaPssSha512GetKlass:
+ *
+ * The RSA-PSS-SHA2-512 signature transform klass.
+ *
+ * Returns: RSA-PSS-SHA2-512 signature transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformRsaPssSha512GetKlass(void) {
+    return(&xmlSecMSCngRsaPssSha512Klass);
+}
+#endif /* XMLSEC_NO_SHA512 */
+
 #endif /* XMLSEC_NO_RSA */
 
-#ifndef XMLSEC_NO_ECDSA
+#ifndef XMLSEC_NO_EC
 
 #ifndef XMLSEC_NO_SHA1
 /****************************************************************************
@@ -954,7 +1326,7 @@ xmlSecMSCngTransformEcdsaSha1GetKlass(void) {
 #ifndef XMLSEC_NO_SHA256
 /****************************************************************************
  *
- * ECDSA-SHA256 signature transform
+ * ECDSA-SHA2-256 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngEcdsaSha256Klass = {
@@ -987,9 +1359,9 @@ static xmlSecTransformKlass xmlSecMSCngEcdsaSha256Klass = {
 /**
  * xmlSecMSCngTransformEcdsaSha256GetKlass:
  *
- * The ECDSA-SHA256 signature transform klass.
+ * The ECDSA-SHA2-256 signature transform klass.
  *
- * Returns: ECDSA-SHA256 signature transform klass.
+ * Returns: ECDSA-SHA2-256 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformEcdsaSha256GetKlass(void) {
@@ -1000,7 +1372,7 @@ xmlSecMSCngTransformEcdsaSha256GetKlass(void) {
 #ifndef XMLSEC_NO_SHA384
 /****************************************************************************
  *
- * ECDSA-SHA384 signature transform
+ * ECDSA-SHA2-384 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngEcdsaSha384Klass = {
@@ -1033,9 +1405,9 @@ static xmlSecTransformKlass xmlSecMSCngEcdsaSha384Klass = {
 /**
  * xmlSecMSCngTransformEcdsaSha384GetKlass:
  *
- * The ECDSA-SHA384 signature transform klass.
+ * The ECDSA-SHA2-384 signature transform klass.
  *
- * Returns: ECDSA-SHA384 signature transform klass.
+ * Returns: ECDSA-SHA2-384 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformEcdsaSha384GetKlass(void) {
@@ -1046,7 +1418,7 @@ xmlSecMSCngTransformEcdsaSha384GetKlass(void) {
 #ifndef XMLSEC_NO_SHA512
 /****************************************************************************
  *
- * ECDSA-SHA512 signature transform
+ * ECDSA-SHA2-512 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCngEcdsaSha512Klass = {
@@ -1079,9 +1451,9 @@ static xmlSecTransformKlass xmlSecMSCngEcdsaSha512Klass = {
 /**
  * xmlSecMSCngTransformEcdsaSha512GetKlass:
  *
- * The ECDSA-SHA512 signature transform klass.
+ * The ECDSA-SHA2-512 signature transform klass.
  *
- * Returns: ECDSA-SHA512 signature transform klass.
+ * Returns: ECDSA-SHA2-512 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCngTransformEcdsaSha512GetKlass(void) {
@@ -1089,4 +1461,4 @@ xmlSecMSCngTransformEcdsaSha512GetKlass(void) {
 }
 #endif /* XMLSEC_NO_SHA512 */
 
-#endif /* XMLSEC_NO_ECDSA */
+#endif /* XMLSEC_NO_EC */
