@@ -117,6 +117,7 @@ xmlSecMSCngKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
         dstCert = CertDuplicateCertificateContext(srcCert);
         if(dstCert == NULL) {
             xmlSecMSCngLastError("CertDuplicateCertificateContext", NULL);
+            CertFreeCertificateContext(srcCert);
             return(-1);
         }
 
@@ -125,6 +126,7 @@ xmlSecMSCngKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
             ret = xmlSecMSCngKeyDataX509AdoptKeyCert(dst, dstCert);
             if (ret < 0) {
                 xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", NULL);
+                CertFreeCertificateContext(srcCert);
                 CertFreeCertificateContext(dstCert);
                 return(-1);
             }
@@ -132,6 +134,7 @@ xmlSecMSCngKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
             ret = xmlSecMSCngKeyDataX509AdoptCert(dst, dstCert);
             if (ret < 0) {
                 xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptCert", NULL);
+                CertFreeCertificateContext(srcCert);
                 CertFreeCertificateContext(dstCert);
                 return(-1);
             }
@@ -290,6 +293,7 @@ xmlSecMSCngKeyDataX509AdoptCrl(xmlSecKeyDataPtr data, PCCRL_CONTEXT crl) {
         xmlSecMSCngLastError("CertAddCRLContextToStore", NULL);
         return(-1);
     }
+    CertFreeCRLContext(crl);
 
     return(0);
 }
@@ -384,8 +388,10 @@ xmlSecMSCnVerifyAndAdoptX509KeyData(xmlSecKeyPtr key, xmlSecKeyDataPtr data, xml
     keyCert = CertDuplicateCertificateContext(cert);
     if(keyCert == NULL) {
         xmlSecMSCngLastError("CertDuplicateCertificateContext", xmlSecKeyDataGetName(data));
+        CertFreeCertificateContext(cert);
         return(-1);
     }
+    CertFreeCertificateContext(cert);
     ret = xmlSecMSCngKeyDataX509AdoptKeyCert(data, keyCert);
     if (ret < 0) {
         xmlSecInternalError("xmlSecMSCngKeyDataX509AdoptKeyCert", xmlSecKeyDataGetName(data));
