@@ -341,6 +341,7 @@ xmlSecMSCryptoKeyDataX509AdoptCrl(xmlSecKeyDataPtr data, PCCRL_CONTEXT crl) {
                             xmlSecKeyDataGetName(data));
         return(-1);
     }
+    CertFreeCRLContext(crl);
     ctx->numCrls++;
 
     return(0);
@@ -449,6 +450,7 @@ xmlSecMSCryptoKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
         if(certDst == NULL) {
             xmlSecMSCryptoError("CertDuplicateCertificateContext",
                                 xmlSecKeyDataGetName(dst));
+            CertFreeCertificateContext(certSrc);
             return(-1);
         }
 
@@ -456,9 +458,11 @@ xmlSecMSCryptoKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
         if(ret < 0) {
             xmlSecInternalError("xmlSecMSCryptoKeyDataX509AdoptCert",
                                 xmlSecKeyDataGetName(dst));
+            CertFreeCertificateContext(certSrc);
             CertFreeCertificateContext(certDst);
             return(-1);
         }
+        CertFreeCertificateContext(certSrc);
     }
 
     /* copy crls */
@@ -476,6 +480,7 @@ xmlSecMSCryptoKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
         if(crlDst == NULL) {
             xmlSecMSCryptoError("CertDuplicateCRLContext",
                                 xmlSecKeyDataGetName(dst));
+            CertFreeCRLContext(crlSrc);
             return(-1);
         }
 
@@ -483,9 +488,11 @@ xmlSecMSCryptoKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
         if(ret < 0) {
             xmlSecInternalError("xmlSecMSCryptoKeyDataX509AdoptCrl",
                                 xmlSecKeyDataGetName(dst));
+            CertFreeCRLContext(crlSrc);
             CertFreeCRLContext(crlDst);
             return(-1);
         }
+        CertFreeCRLContext(crlSrc);
     }
 
     /* copy key cert if exist */
@@ -523,7 +530,7 @@ xmlSecMSCryptoKeyDataX509Finalize(xmlSecKeyDataPtr data) {
     }
 
     if (ctx->hMemStore != 0) {
-        if (!CertCloseStore(ctx->hMemStore, CERT_CLOSE_STORE_FORCE_FLAG)) {
+        if (!CertCloseStore(ctx->hMemStore, 0)) {
             xmlSecInternalError("CertCloseStore", NULL);
             return;
         }
@@ -640,6 +647,7 @@ xmlSecMSCryptoKeyDataX509DebugDump(xmlSecKeyDataPtr data, FILE* output) {
         }
         fprintf(output, "==== Certificate:\n");
         xmlSecMSCryptoX509CertDebugDump(cert, output);
+        CertFreeCertificateContext(cert);
     }
 
     /* we don't print out crls */
@@ -673,6 +681,7 @@ xmlSecMSCryptoKeyDataX509DebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
         fprintf(output, "<Certificate>\n");
         xmlSecMSCryptoX509CertDebugXmlDump(cert, output);
         fprintf(output, "</Certificate>\n");
+        CertFreeCertificateContext(cert);
     }
 
     /* we don't print out crls */
@@ -820,6 +829,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT "; certSize=%lu",
                     ctx->crtPos, cert->cbCertEncoded);
+                CertFreeCertificateContext(cert);
                 return(-1);
             }
         }
@@ -829,6 +839,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                 xmlSecInternalError2("xmlSecMSCryptoX509SKIWrite",
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
+                CertFreeCertificateContext(cert);
                 return(-1);
             }
         }
@@ -841,6 +852,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                 xmlSecInternalError2("xmlSecMSCryptoX509NameWrite(subject)",
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
+                CertFreeCertificateContext(cert);
                 return(-1);
             }
         }
@@ -854,6 +866,7 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                 xmlSecInternalError2("xmlSecMSCryptoX509NameWrite(issuer name)",
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
+                CertFreeCertificateContext(cert);
                 return(-1);
             }
             x509Value->issuerSerial = xmlSecMSCryptoASN1IntegerWrite(&(cert->pCertInfo->SerialNumber));
@@ -861,8 +874,10 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                 xmlSecInternalError2("xmlSecMSCryptoASN1IntegerWrite(issuer serial))",
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT, ctx->crtPos);
+                CertFreeCertificateContext(cert);
                 return(-1);
             }
+            CertFreeCertificateContext(cert);
         }
         ++ctx->crtPos;
     }
@@ -883,9 +898,11 @@ xmlSecMSCryptoKeyDataX509Write(xmlSecKeyDataPtr data, xmlSecKeyX509DataValuePtr
                     xmlSecKeyDataGetName(data),
                     "pos=" XMLSEC_SIZE_FMT "; crlSize=%lu",
                     ctx->crlPos, crl->cbCrlEncoded);
+                CertFreeCRLContext(crl);
                 return(-1);
             }
         }
+        CertFreeCRLContext(crl);
         ++ctx->crlPos;
     }
     else {
@@ -932,8 +949,10 @@ xmlSecMSCryptoKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data, xmlSecKeyPtr
             if(ctx->keyCert == NULL) {
                     xmlSecMSCryptoError("CertDuplicateCertificateContext",
                                         xmlSecKeyDataGetName(data));
+                    CertFreeCertificateContext(cert);
                     return(-1);
             }
+            CertFreeCertificateContext(cert);
 
                 /* search key according to KeyReq */
                 pCert = CertDuplicateCertificateContext( ctx->keyCert ) ;
@@ -961,6 +980,9 @@ xmlSecMSCryptoKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data, xmlSecKeyPtr
                                 return(-1);
                         }
                         pCert = NULL ;
+                } else {
+                        CertFreeCertificateContext(pCert);
+                        pCert = NULL;
                 }
 
             /* verify that the key matches our expectations */
