@@ -1,11 +1,12 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Application support functions for NSS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2003 America Online, Inc.  All rights reserved.
  */
 /**
@@ -13,6 +14,7 @@
  * @Short_description: Application support functions for NSS.
  * @Stability: Stable
  *
+ * Common functions for xmlsec1 command line utility tool for NSS.
  */
 
 #include "globals.h"
@@ -21,9 +23,10 @@
 
 #include <nspr.h>
 #include <nss.h>
-#include <pk11pub.h>
 #include <cert.h>
+#include <certdb.h>
 #include <keyhi.h>
+#include <pk11func.h>
 #include <pkcs12.h>
 #include <p12plcy.h>
 /*
@@ -34,6 +37,7 @@
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/nss/app.h>
 #include <xmlsec/nss/crypto.h>
@@ -42,13 +46,8 @@
 #include <xmlsec/nss/keysstore.h>
 
 #include "../cast_helpers.h"
+#include "private.h"
 
-/* workaround - NSS exports this but doesn't declare it */
-extern CERTCertificate * __CERT_NewTempCertificate              (CERTCertDBHandle *handle,
-                                                                 SECItem *derCert,
-                                                                 char *nickname,
-                                                                 PRBool isperm,
-                                                                 PRBool copyDER);
 static int xmlSecNssAppCreateSECItem                            (SECItem *contents,
                                                                  const xmlSecByte* data,
                                                                  xmlSecSize dataSize);
@@ -61,10 +60,14 @@ static PRBool xmlSecNssAppAscii2UCS2Conv                        (PRBool toUnicod
                                                                  unsigned int   maxOutBufLen,
                                                                  unsigned int  *outBufLen,
                                                                  PRBool         swapBytes);
+static xmlSecKeyPtr     xmlSecNssAppDerKeyLoadSECItem           (SECItem* secItem);
+
+
+#ifndef XMLSEC_NO_X509
 static SECItem *xmlSecNssAppNicknameCollisionCallback           (SECItem *old_nick,
                                                                  PRBool *cancel,
                                                                  void *wincx);
-static xmlSecKeyPtr     xmlSecNssAppDerKeyLoadSECItem           (SECItem* secItem);
+#endif /* XMLSEC_NO_X509 */
 
 /**
  * xmlSecNssAppInit:
@@ -234,19 +237,40 @@ xmlSecNssAppAscii2UCS2Conv(PRBool toUnicode,
                                     outBuf, maxOutBufLen, outBufLen));
 }
 
+#ifndef XMLSEC_NO_X509
+/* rename certificate if needed */
 static SECItem *
 xmlSecNssAppNicknameCollisionCallback(SECItem *old_nick ATTRIBUTE_UNUSED,
-                                      PRBool *cancel,
-                                      void *wincx ATTRIBUTE_UNUSED)
-{
-    if (cancel == NULL) {
-        return (NULL);
+    PRBool *cancel, void *wincx ATTRIBUTE_UNUSED
+) {
+    CERTCertificate *cert = (CERTCertificate *)wincx;
+    char *nick = NULL;
+    SECItem *ret_nick = NULL;
+
+    if((cancel  == NULL) || (cert == NULL)) {
+        xmlSecNssError("cert is missing", NULL);
+        return(NULL);
     }
 
-    /* XXX not handled yet  */
-    *cancel = PR_TRUE;
-    return (NULL);
+    nick = CERT_MakeCANickname(cert);
+    if (!nick) {
+        xmlSecNssError("CERT_MakeCANickname", NULL);
+        return(NULL);
+    }
+
+    ret_nick = PORT_ZNew(SECItem);
+    if (ret_nick == NULL) {
+        xmlSecNssError("PORT_ZNew", NULL);
+        PORT_Free(nick);
+        return NULL;
+    }
+
+    /* done */
+    ret_nick->data = (unsigned char *)nick;
+    ret_nick->len = (unsigned int)PORT_Strlen(nick);
+    return ret_nick;
 }
+#endif /* XMLSEC_NO_X509 */
 
 /**
  * xmlSecNssAppKeyLoad:
@@ -256,19 +280,41 @@ xmlSecNssAppNicknameCollisionCallback(SECItem *old_nick ATTRIBUTE_UNUSED,
  * @pwdCallback:        the key password callback.
  * @pwdCallbackCtx:     the user context for password callback.
  *
+ * Deprecated, use @xmlSecNssAppKeyLoadEx instead. Reads key from a file
+ *
+ * Returns: pointer to the key or NULL if an error occurs.
+ */
+xmlSecKeyPtr
+xmlSecNssAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
+    return(xmlSecNssAppKeyLoadEx(filename, xmlSecKeyDataTypeUnknown, format, pwd, pwdCallback, pwdCallbackCtx));
+}
+
+/**
+ * xmlSecNssAppKeyLoadEx:
+ * @filename:           the key filename.
+ * @type:               the key type (public / private).
+ * @format:             the key file format.
+ * @pwd:                the key file password.
+ * @pwdCallback:        the key password callback.
+ * @pwdCallbackCtx:     the user context for password callback.
+ *
  * Reads key from a file
  *
  * Returns: pointer to the key or NULL if an error occurs.
  */
 xmlSecKeyPtr
-xmlSecNssAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
-                    const char *pwd, void* pwdCallback, void* pwdCallbackCtx) {
+xmlSecNssAppKeyLoadEx(const char *filename, xmlSecKeyDataType type ATTRIBUTE_UNUSED, xmlSecKeyDataFormat format,
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     SECItem secItem;
     xmlSecKeyPtr res;
     int ret;
 
     xmlSecAssert2(filename != NULL, NULL);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, NULL);
+    UNREFERENCED_PARAMETER(type);
 
     /* read the file contents */
     memset(&secItem, 0, sizeof(secItem));
@@ -344,9 +390,8 @@ xmlSecNssAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecKey
  */
 xmlSecKeyPtr
 xmlSecNssAppKeyLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format,
-                    const char *pwd,
-                    void* pwdCallback,
-                    void* pwdCallbackCtx) {
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     xmlSecKeyPtr key = NULL;
 
     xmlSecAssert2(secItem != NULL, NULL);
@@ -415,7 +460,7 @@ xmlSecNssAppDerKeyLoadSECItem(SECItem* secItem) {
 
     /* TRY PRIVATE KEY FIRST
      * Note: This expects the key to be in PrivateKeyInfo format. The
-     * DER files created from PEM via openssl utilities aren't in that
+     * DER files created from PEM via nss utilities aren't in that
      * format
      */
     status = PK11_ImportDERPrivateKeyInfoAndReturnKey(slot, secItem,
@@ -426,6 +471,7 @@ xmlSecNssAppDerKeyLoadSECItem(SECItem* secItem) {
         spki = SECKEY_DecodeDERSubjectPublicKeyInfo(secItem);
         if (spki == NULL) {
             xmlSecNssError("SECKEY_DecodeDERSubjectPublicKeyInfo", NULL);
+            goto done;
         }
 
         pubkey = SECKEY_ExtractPublicKey(spki);
@@ -573,61 +619,56 @@ xmlSecNssAppKeyCertLoadMemory(xmlSecKeyPtr key, const xmlSecByte* data, xmlSecSi
 int
 xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyDataFormat format) {
     CERTCertificate *cert = NULL;
-    CERTCertificate *keyCert = NULL;
-    xmlSecKeyDataPtr data;
+    xmlSecKeyDataPtr x509Data;
     int ret;
     int res = -1;
 
     xmlSecAssert2(key != NULL, -1);
     xmlSecAssert2(secItem != NULL, -1);
+    xmlSecAssert2(secItem->type == siBuffer, -1);
+    xmlSecAssert2(secItem->data != NULL, -1);
+    xmlSecAssert2(secItem->len > 0, -1);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
 
-    data = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
-    if(data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData(xmlSecNssKeyDataX509Id)", NULL);
-        goto done;
-    }
-
     /* read cert */
     switch(format) {
     case xmlSecKeyDataFormatPkcs8Der:
     case xmlSecKeyDataFormatDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                         secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", xmlSecKeyDataGetName(data),
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
                 "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
             goto done;
         }
         break;
     default:
-        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, xmlSecKeyDataGetName(data),
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
             "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
         goto done;
     }
     xmlSecAssert2(cert != NULL, -1);
 
-    /* make a copy for key cert */
-    keyCert = CERT_DupCertificate(cert);
-    if(keyCert == NULL) {
-        xmlSecNssError("CERT_DupCertificate", xmlSecKeyDataGetName(data));
+    /* add cert to the key */
+    x509Data = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
+    if(x509Data == NULL) {
+        xmlSecInternalError("xmlSecKeyEnsureData(xmlSecNssKeyDataX509Id)", NULL);
         goto done;
     }
-
-    /* add both cert and key cert in the data */
-    ret = xmlSecNssKeyDataX509AdoptCert(data, cert);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", xmlSecKeyDataGetName(data));
-        goto done;
+    if(xmlSecNssKeyDataX509GetKeyCert(x509Data) == NULL) {
+        /* TODO: check if cert matches the key */
+        ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
+            goto done;
+        }
+    } else {
+        ret = xmlSecNssKeyDataX509AdoptCert(x509Data, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
+            goto done;
+        }
     }
-    cert = NULL; /* owned by data now */
-
-    ret = xmlSecNssKeyDataX509AdoptKeyCert(data, keyCert);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", xmlSecKeyDataGetName(data));
-        goto done;
-    }
-    keyCert = NULL; /* owned by data now */
+    cert = NULL; /* owned by x509Data now */
 
     /* success */
     res = 0;
@@ -636,9 +677,6 @@ done:
     if(cert != NULL) {
         CERT_DestroyCertificate(cert);
     }
-    if(keyCert != NULL) {
-        CERT_DestroyCertificate(keyCert);
-    }
     return(res);
 }
 
@@ -650,7 +688,7 @@ done:
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 file.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -693,7 +731,7 @@ xmlSecNssAppPkcs12Load(const char *filename, const char *pwd,
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 binary data.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -735,7 +773,7 @@ xmlSecNssAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, const
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 SECItem.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -745,7 +783,7 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
                        void *pwdCallback ATTRIBUTE_UNUSED,
                        void* pwdCallbackCtx ATTRIBUTE_UNUSED) {
     xmlSecKeyPtr key = NULL;
-    xmlSecKeyDataPtr data = NULL;
+    xmlSecKeyDataPtr keyValueData = NULL;
     xmlSecKeyDataPtr x509Data = NULL;
     int ret;
     PK11SlotInfo *slot = NULL;
@@ -759,7 +797,9 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
     CERTCertificate     *cert = NULL;
     CERTCertificate     *tmpcert = NULL;
     SEC_PKCS12DecoderContext *p12ctx = NULL;
+    const SEC_PKCS12DecoderItem *dip;
     size_t pwdSize;
+    xmlSecKeyPtr res = NULL;
 
     xmlSecAssert2((secItem != NULL), NULL);
 
@@ -832,8 +872,7 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
 
     x509Data = xmlSecKeyDataCreate(xmlSecNssKeyDataX509Id);
     if(x509Data == NULL) {
-        xmlSecInternalError("xmlSecKeyDataCreate",
-                            xmlSecTransformKlassGetName(xmlSecNssKeyDataX509Id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecNssKeyDataX509Id)", NULL);
         goto done;
     }
 
@@ -842,96 +881,112 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
         privkey = PK11_FindKeyByAnyCert(cert, NULL);
 
         if (privkey != NULL) {
-            if (data != NULL) {
+            if (keyValueData != NULL) {
                 /* we already found a private key.
                  * assume the first private key we find is THE ONE
                  */
                 SECKEY_DestroyPrivateKey(privkey);
                 privkey = NULL;
-            } else {
-                pubkey = CERT_ExtractPublicKey(cert);
-                if (pubkey == NULL) {
-                    xmlSecNssError("CERT_ExtractPublicKey",
-                                   xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-                data = xmlSecNssPKIAdoptKey(privkey, pubkey);
-                if(data == NULL) {
-                    xmlSecInternalError("xmlSecNssPKIAdoptKey",
-                                        xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-
-                pubkey = NULL;
-                privkey = NULL;
-
-                tmpcert = CERT_DupCertificate(cert);
-                if(tmpcert == NULL) {
-                    xmlSecNssError("CERT_DupCertificate",
-                                   xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-
-                ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, tmpcert);
-                if(ret < 0) {
-                    xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert",
-                                        xmlSecKeyDataGetName(x509Data));
-                    CERT_DestroyCertificate(tmpcert);
-                    goto done;
-                }
-
+                continue;
             }
-        }
 
-        tmpcert = CERT_DupCertificate(cert);
-        if(tmpcert == NULL) {
-            xmlSecNssError("CERT_DupCertificate",
-                           xmlSecKeyDataGetName(x509Data));
-            goto done;
-        }
-        ret = xmlSecNssKeyDataX509AdoptCert(x509Data, tmpcert);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert",
-                                xmlSecKeyDataGetName(x509Data));
-            CERT_DestroyCertificate(tmpcert);
-            goto done;
-        }
+            pubkey = CERT_ExtractPublicKey(cert);
+            if (pubkey == NULL) {
+                xmlSecNssError("CERT_ExtractPublicKey", NULL);
+                goto done;
+            }
+            keyValueData = xmlSecNssPKIAdoptKey(privkey, pubkey);
+            if(keyValueData == NULL) {
+                xmlSecInternalError("xmlSecNssPKIAdoptKey", NULL);
+                goto done;
+            }
 
+            pubkey = NULL;
+            privkey = NULL;
+
+            tmpcert = CERT_DupCertificate(cert);
+            if(tmpcert == NULL) {
+                xmlSecNssError("CERT_DupCertificate", NULL);
+                goto done;
+            }
+
+            ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, tmpcert);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
+                CERT_DestroyCertificate(tmpcert);
+                goto done;
+            }
+            tmpcert = NULL; /* owned by x509Data now */
+        } else {
+            tmpcert = CERT_DupCertificate(cert);
+            if(tmpcert == NULL) {
+                xmlSecNssError("CERT_DupCertificate", NULL);
+                goto done;
+            }
+            ret = xmlSecNssKeyDataX509AdoptCert(x509Data, tmpcert);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
+                CERT_DestroyCertificate(tmpcert);
+                goto done;
+            }
+            tmpcert = NULL; /* owned by x509Data now */
+        }
     } /* end for loop */
 
-    if (data == NULL) {
+    if (keyValueData == NULL) {
         /* private key not found in PKCS12 file */
         xmlSecInternalError("xmlSecNssAppPkcs12Load(private key)", NULL);
         goto done;
      }
 
+    /* create key and set key value and x509 data into it */
     key = xmlSecKeyCreate();
     if(key == NULL) {
         xmlSecInternalError("xmlSecKeyCreate", NULL);
         goto done;
     }
-
-    ret = xmlSecKeySetValue(key, data);
+    ret = xmlSecKeySetValue(key, keyValueData);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecKeySetValue",
-                            xmlSecKeyDataGetName(x509Data));
-        xmlSecKeyDestroy(key);
-        key = NULL;
+        xmlSecInternalError("xmlSecKeySetValue", NULL);
         goto done;
     }
-    data = NULL;
+    keyValueData = NULL; /* owned by key now */
 
     ret = xmlSecKeyAdoptData(key, x509Data);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecKeyAdoptData",
-                            xmlSecKeyDataGetName(x509Data));
-        xmlSecKeyDestroy(key);
-        key = NULL;
+        xmlSecInternalError("xmlSecKeyAdoptData", NULL);
         goto done;
     }
-    x509Data = NULL;
+    x509Data = NULL; /* owned by key now */
+
+    /* try to find key name */
+    rv = SEC_PKCS12DecoderIterateInit(p12ctx);
+    if (rv != SECSuccess) {
+        xmlSecNssError("SEC_PKCS12DecoderIterateInit", NULL);
+        goto done;
+    }
+    /* read pkcs12 bags */
+    while (SEC_PKCS12DecoderIterateNext(p12ctx, &dip) == SECSuccess) {
+         if((dip->friendlyName != NULL) && (dip->friendlyName->data != NULL) && (dip->friendlyName->len > 0) ) {
+            ret = xmlSecKeySetNameEx(key, dip->friendlyName->data, dip->friendlyName->len);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecKeySetNameEx", NULL);
+                goto done;
+            }
+            /* use the first one */
+            break;
+         }
+    }
+
+
+    /* success */
+    res = key;
+    key = NULL;
 
 done:
+    if(key != NULL) {
+        xmlSecKeyDestroy(key);
+    }
     if (p12ctx) {
         SEC_PKCS12DecoderFinish(p12ctx);
     }
@@ -945,8 +1000,8 @@ done:
     if(x509Data != NULL) {
         xmlSecKeyDataDestroy(x509Data);
     }
-    if(data != NULL) {
-        xmlSecKeyDataDestroy(data);
+    if(keyValueData != NULL) {
+        xmlSecKeyDataDestroy(keyValueData);
     }
     if (privkey) {
         SECKEY_DestroyPrivateKey(privkey);
@@ -955,7 +1010,7 @@ done:
         SECKEY_DestroyPublicKey(pubkey);
     }
 
-    return(key);
+    return(res);
 }
 
 /**
@@ -973,20 +1028,21 @@ xmlSecNssAppKeyFromCertLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format)
     xmlSecKeyDataPtr keyData = NULL;
     xmlSecKeyDataPtr certData;
     CERTCertificate *cert = NULL;
-    CERTCertificate *keyCert = NULL;
     int ret;
     xmlSecKeyPtr res = NULL;
 
     xmlSecAssert2(secItem != NULL, NULL);
+    xmlSecAssert2(secItem->type == siBuffer, NULL);
+    xmlSecAssert2(secItem->data != NULL, NULL);
+    xmlSecAssert2(secItem->len > 0, NULL);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, NULL);
 
     /* load cert */
     switch(format) {
     case xmlSecKeyDataFormatCertDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                    secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", NULL,
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
                 "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
             goto done;
         }
@@ -1004,21 +1060,12 @@ xmlSecNssAppKeyFromCertLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format)
         goto done;
     }
 
-    /* create key */
+    /* create key set key value */
     key = xmlSecKeyCreate();
     if(key == NULL) {
         xmlSecInternalError("xmlSecKeyCreate", NULL);
         goto done;
     }
-
-    /* make a copy for key cert */
-    keyCert = CERT_DupCertificate(cert);
-    if(keyCert == NULL) {
-        xmlSecNssError("CERT_DupCertificate", NULL);
-        goto done;
-    }
-
-    /* set key value */
     ret = xmlSecKeySetValue(key, keyData);
     if(ret < 0) {
         xmlSecInternalError("xmlSecKeySetValue", NULL);
@@ -1026,33 +1073,23 @@ xmlSecNssAppKeyFromCertLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format)
     }
     keyData = NULL; /* owned by key now */
 
-    /* create cert data */
+    /* create cert data put key's cert into it */
     certData = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
     if(certData == NULL) {
         xmlSecInternalError("xmlSecKeyEnsureData", NULL);
         goto done;
     }
-
-    /* put cert and key cert in the cert data */
-    ret = xmlSecNssKeyDataX509AdoptCert(certData, cert);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
-        goto done;
-    }
-    cert = NULL; /* owned by data now */
-
-    ret = xmlSecNssKeyDataX509AdoptKeyCert(certData, keyCert);
+    ret = xmlSecNssKeyDataX509AdoptKeyCert(certData, cert);
     if(ret < 0) {
         xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
         goto done;
     }
-    keyCert = NULL; /* owned by data now */
+    cert = NULL; /* owned by data now */
 
     /* success */
     res = key;
     key = NULL;
 
-
 done:
     if(key != NULL) {
         xmlSecKeyDestroy(key);
@@ -1063,14 +1100,9 @@ done:
     if(cert != NULL) {
         CERT_DestroyCertificate(cert);
     }
-    if(keyCert != NULL) {
-        CERT_DestroyCertificate(keyCert);
-    }
-
     return(res);
 }
 
-
 /**
  * xmlSecNssAppKeysMngrCertLoad:
  * @mngr:               the pointer to keys manager.
@@ -1116,8 +1148,8 @@ xmlSecNssAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
 /**
  * xmlSecNssAppKeysMngrCertLoadMemory:
  * @mngr:               the pointer to keys manager.
- * @data:               the key binary data.
- * @dataSize:           the key binary data size.
+ * @data:               the certificatedata.
+ * @dataSize:           the certificate data size.
  * @format:             the certificate format (PEM or DER).
  * @type:               the certificate type (trusted/untrusted).
  *
@@ -1177,6 +1209,9 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
 
     xmlSecAssert2(mngr != NULL, -1);
     xmlSecAssert2(secItem != NULL, -1);
+    xmlSecAssert2(secItem->type == siBuffer, -1);
+    xmlSecAssert2(secItem->data != NULL, -1);
+    xmlSecAssert2(secItem->len > 0, -1);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
 
     x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
@@ -1187,10 +1222,9 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
 
     switch(format) {
     case xmlSecKeyDataFormatDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                         secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", NULL,
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
                 "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
             return(-1);
         }
@@ -1211,6 +1245,141 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
     return(0);
 }
 
+/**
+ * xmlSecNssAppKeysMngrCrlLoad:
+ * @mngr:               the pointer to keys manager.
+ * @filename:           the CRL file.
+ * @format:             the CRL file format (PEM or DER).
+ *
+ * Reads crl from @filename and adds to the list of crls in @store
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecNssAppKeysMngrCrlLoad(xmlSecKeysMngrPtr mngr, const char *filename, xmlSecKeyDataFormat format) {
+    xmlSecKeyDataStorePtr x509Store;
+    CERTSignedCrl* crl;
+    SECItem secItem;
+    int ret;
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(filename != NULL, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    /* read the file contents */
+    memset(&secItem, 0, sizeof(secItem));
+    ret = xmlSecNssAppReadSECItem(&secItem, filename);
+    if((ret < 0) || (secItem.type != siBuffer) ||(secItem.data == NULL) || (secItem.len <= 0)) {
+        xmlSecInternalError("xmlSecNssAppReadSECItem", NULL);
+        return(-1);
+    }
+
+    /* read CRL */
+    switch(format) {
+    case xmlSecKeyDataFormatDer:
+        crl = xmlSecNssX509CrlDerRead(secItem.data, secItem.len, XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS);
+        if(crl == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CrlDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            SECITEM_FreeItem(&secItem, PR_FALSE);
+            return(-1);
+        }
+        break;
+    default:
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        SECITEM_FreeItem(&secItem, PR_FALSE);
+        return(-1);
+    }
+    SECITEM_FreeItem(&secItem, PR_FALSE);
+
+    /* Add CRL to the store */
+    ret = xmlSecNssX509StoreAdoptCrl(x509Store, crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509StoreAdoptCrl", NULL);
+        SEC_DestroyCrl(crl);
+        return(-1);
+    }
+    crl = NULL; /* owned by x509data now */
+
+    /* done */
+    return(0);
+}
+
+/**
+ * xmlSecNssAppKeysMngrCrlLoadMemory:
+ * @mngr:               the pointer to keys manager.
+ * @data:               the CRL data.
+ * @dataSize:           the CRL data size.
+ * @format:             the CRL format (PEM or DER).
+ *
+ * Reads crl from @data and adds to the list of crls in @store
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecNssAppKeysMngrCrlLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format) {
+    xmlSecKeyDataStorePtr x509Store;
+    CERTSignedCrl* crl;
+    SECItem secItem;
+    int ret;
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    memset(&secItem, 0, sizeof(secItem));
+    ret = xmlSecNssAppCreateSECItem(&secItem, data, dataSize);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssAppCreateSECItem", NULL);
+        return(-1);
+    }
+
+    /* read CRL */
+    switch(format) {
+    case xmlSecKeyDataFormatDer:
+        crl = xmlSecNssX509CrlDerRead(secItem.data, secItem.len, XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS);
+        if(crl == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CrlDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            SECITEM_FreeItem(&secItem, PR_FALSE);
+            return(-1);
+        }
+        break;
+    default:
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        SECITEM_FreeItem(&secItem, PR_FALSE);
+        return(-1);
+    }
+    SECITEM_FreeItem(&secItem, PR_FALSE);
+
+    /* Add CRL to the store */
+    ret = xmlSecNssX509StoreAdoptCrl(x509Store, crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509StoreAdoptCrl", NULL);
+        SEC_DestroyCrl(crl);
+        return(-1);
+    }
+    crl = NULL; /* owned by x509data now */
+
+    /* done */
+    return(0);
+}
+
+
 #endif /* XMLSEC_NO_X509 */
 
 /**
@@ -1289,6 +1458,33 @@ xmlSecNssAppDefaultKeysMngrAdoptKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key) {
     return(0);
 }
 
+/**
+ * xmlSecNssAppDefaultKeysMngrVerifyKey:
+ * @mngr:               the pointer to keys manager.
+ * @key:                the pointer to key.
+ * @keyInfoCtx:         the key info context for verification.
+ *
+ * Verifies @key with the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function:
+ * - Checks that key certificate is present
+ * - Checks that key certificate is valid
+ *
+ * Adds @key to the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function.
+ *
+ * Returns: 1 if key is verified, 0 otherwise, or a negative value if an error occurs.
+ */
+int
+xmlSecNssAppDefaultKeysMngrVerifyKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    xmlSecNotImplementedError("xmlSecNssAppDefaultKeysMngrVerifyKey");
+    return(-1);
+}
+
 /**
  * xmlSecNssAppDefaultKeysMngrLoad:
  * @mngr:               the pointer to keys manager.
@@ -1368,4 +1564,3 @@ void*
 xmlSecNssAppGetDefaultPwdCallback(void) {
     return(NULL);
 }
-
