@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Digests transforms implementation for OpenSSL.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:digests
- * @Short_description: Digests transforms implementation for OpenSSL.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -118,6 +116,21 @@ xmlSecOpenSSLEvpDigestCheckId(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
+#ifndef XMLSEC_NO_SHA3
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_224Id)) {
+        return(1);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_256Id)) {
+        return(1);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_384Id)) {
+        return(1);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_512Id)) {
+        return(1);
+    } else
+#endif /* XMLSEC_NO_SHA3 */
+
 #ifndef XMLSEC_NO_GOST
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformGostR3411_94Id)) {
         return(1);
@@ -237,6 +250,21 @@ xmlSecOpenSSLEvpDigestInitialize(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
+#ifndef XMLSEC_NO_SHA3
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_224Id)) {
+        XMLSEC_OPENSSL_EVP_DIGEST_SETUP(ctx, EVP_sha3_224(), OSSL_DIGEST_NAME_SHA3_224);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_256Id)) {
+        XMLSEC_OPENSSL_EVP_DIGEST_SETUP(ctx, EVP_sha3_256(), OSSL_DIGEST_NAME_SHA3_256);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_384Id)) {
+        XMLSEC_OPENSSL_EVP_DIGEST_SETUP(ctx, EVP_sha3_384(), OSSL_DIGEST_NAME_SHA3_384);
+    } else
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformSha3_512Id)) {
+        XMLSEC_OPENSSL_EVP_DIGEST_SETUP(ctx, EVP_sha3_512(), OSSL_DIGEST_NAME_SHA3_512);
+    } else
+#endif /* XMLSEC_NO_SHA3 */
+
 #ifndef XMLSEC_NO_GOST
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformGostR3411_94Id)) {
         int ret;
@@ -603,7 +631,7 @@ xmlSecOpenSSLTransformSha1GetKlass(void) {
 #ifndef XMLSEC_NO_SHA224
 /******************************************************************************
  *
- * SHA224
+ * SHA2-224
  *
  *****************************************************************************/
 static xmlSecTransformKlass xmlSecOpenSSLSha224Klass = {
@@ -636,9 +664,9 @@ static xmlSecTransformKlass xmlSecOpenSSLSha224Klass = {
 /**
  * xmlSecOpenSSLTransformSha224GetKlass:
  *
- * SHA-224 digest transform klass.
+ * SHA2-224 digest transform klass.
  *
- * Returns: pointer to SHA-224 digest transform klass.
+ * Returns: pointer to SHA2-224 digest transform klass.
  */
 xmlSecTransformId
 xmlSecOpenSSLTransformSha224GetKlass(void) {
@@ -649,7 +677,7 @@ xmlSecOpenSSLTransformSha224GetKlass(void) {
 #ifndef XMLSEC_NO_SHA256
 /******************************************************************************
  *
- * SHA256
+ * SHA2-256
  *
  *****************************************************************************/
 static xmlSecTransformKlass xmlSecOpenSSLSha256Klass = {
@@ -682,9 +710,9 @@ static xmlSecTransformKlass xmlSecOpenSSLSha256Klass = {
 /**
  * xmlSecOpenSSLTransformSha256GetKlass:
  *
- * SHA-256 digest transform klass.
+ * SHA2-256 digest transform klass.
  *
- * Returns: pointer to SHA-256 digest transform klass.
+ * Returns: pointer to SHA2-256 digest transform klass.
  */
 xmlSecTransformId
 xmlSecOpenSSLTransformSha256GetKlass(void) {
@@ -695,7 +723,7 @@ xmlSecOpenSSLTransformSha256GetKlass(void) {
 #ifndef XMLSEC_NO_SHA384
 /******************************************************************************
  *
- * SHA384
+ * SHA2-384
  *
  *****************************************************************************/
 static xmlSecTransformKlass xmlSecOpenSSLSha384Klass = {
@@ -728,9 +756,9 @@ static xmlSecTransformKlass xmlSecOpenSSLSha384Klass = {
 /**
  * xmlSecOpenSSLTransformSha384GetKlass:
  *
- * SHA-384 digest transform klass.
+ * SHA2-384 digest transform klass.
  *
- * Returns: pointer to SHA-384 digest transform klass.
+ * Returns: pointer to SHA2-384 digest transform klass.
  */
 xmlSecTransformId
 xmlSecOpenSSLTransformSha384GetKlass(void) {
@@ -741,7 +769,7 @@ xmlSecOpenSSLTransformSha384GetKlass(void) {
 #ifndef XMLSEC_NO_SHA512
 /******************************************************************************
  *
- * SHA512
+ * SHA2-512
  *
  *****************************************************************************/
 static xmlSecTransformKlass xmlSecOpenSSLSha512Klass = {
@@ -774,9 +802,9 @@ static xmlSecTransformKlass xmlSecOpenSSLSha512Klass = {
 /**
  * xmlSecOpenSSLTransformSha512GetKlass:
  *
- * SHA-512 digest transform klass.
+ * SHA2-512 digest transform klass.
  *
- * Returns: pointer to SHA-512 digest transform klass.
+ * Returns: pointer to SHA2-512 digest transform klass.
  */
 xmlSecTransformId
 xmlSecOpenSSLTransformSha512GetKlass(void) {
@@ -784,6 +812,185 @@ xmlSecOpenSSLTransformSha512GetKlass(void) {
 }
 #endif /* XMLSEC_NO_SHA512 */
 
+
+#ifndef XMLSEC_NO_SHA3
+/******************************************************************************
+ *
+ * SHA3-224
+ *
+ *****************************************************************************/
+static xmlSecTransformKlass xmlSecOpenSSLSha3_224Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSecOpenSSLEvpDigestSize,                 /* xmlSecSize objSize */
+
+    xmlSecNameSha3_224,                         /* const xmlChar* name; */
+    xmlSecHrefSha3_224,                         /* const xmlChar* href; */
+    xmlSecTransformUsageDigestMethod,           /* xmlSecTransformUsage usage; */
+
+    xmlSecOpenSSLEvpDigestInitialize,           /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecOpenSSLEvpDigestFinalize,             /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    NULL,                                       /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    NULL,                                       /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecOpenSSLEvpDigestVerify,               /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecOpenSSLEvpDigestExecute,              /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecOpenSSLTransformSha3_224GetKlass:
+ *
+ * SHA3-224 digest transform klass.
+ *
+ * Returns: pointer to SHA3-224 digest transform klass.
+ */
+xmlSecTransformId
+xmlSecOpenSSLTransformSha3_224GetKlass(void) {
+    return(&xmlSecOpenSSLSha3_224Klass);
+}
+
+/******************************************************************************
+ *
+ * SHA3-256
+ *
+ *****************************************************************************/
+static xmlSecTransformKlass xmlSecOpenSSLSha3_256Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSecOpenSSLEvpDigestSize,                 /* xmlSecSize objSize */
+
+    xmlSecNameSha3_256,                         /* const xmlChar* name; */
+    xmlSecHrefSha3_256,                         /* const xmlChar* href; */
+    xmlSecTransformUsageDigestMethod,           /* xmlSecTransformUsage usage; */
+
+    xmlSecOpenSSLEvpDigestInitialize,           /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecOpenSSLEvpDigestFinalize,             /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    NULL,                                       /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    NULL,                                       /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecOpenSSLEvpDigestVerify,               /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecOpenSSLEvpDigestExecute,              /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecOpenSSLTransformSha3_256GetKlass:
+ *
+ * SHA3-256 digest transform klass.
+ *
+ * Returns: pointer to SHA3-256 digest transform klass.
+ */
+xmlSecTransformId
+xmlSecOpenSSLTransformSha3_256GetKlass(void) {
+    return(&xmlSecOpenSSLSha3_256Klass);
+}
+
+/******************************************************************************
+ *
+ * SHA3-384
+ *
+ *****************************************************************************/
+static xmlSecTransformKlass xmlSecOpenSSLSha3_384Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSecOpenSSLEvpDigestSize,                 /* xmlSecSize objSize */
+
+    xmlSecNameSha3_384,                         /* const xmlChar* name; */
+    xmlSecHrefSha3_384,                         /* const xmlChar* href; */
+    xmlSecTransformUsageDigestMethod,           /* xmlSecTransformUsage usage; */
+
+    xmlSecOpenSSLEvpDigestInitialize,           /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecOpenSSLEvpDigestFinalize,             /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    NULL,                                       /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    NULL,                                       /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecOpenSSLEvpDigestVerify,               /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecOpenSSLEvpDigestExecute,              /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecOpenSSLTransformSha3_384GetKlass:
+ *
+ * SHA3-384 digest transform klass.
+ *
+ * Returns: pointer to SHA3-384 digest transform klass.
+ */
+xmlSecTransformId
+xmlSecOpenSSLTransformSha3_384GetKlass(void) {
+    return(&xmlSecOpenSSLSha3_384Klass);
+}
+
+/******************************************************************************
+ *
+ * SHA3-512
+ *
+ *****************************************************************************/
+static xmlSecTransformKlass xmlSecOpenSSLSha3_512Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSecOpenSSLEvpDigestSize,                 /* xmlSecSize objSize */
+
+    xmlSecNameSha3_512,                         /* const xmlChar* name; */
+    xmlSecHrefSha3_512,                         /* const xmlChar* href; */
+    xmlSecTransformUsageDigestMethod,           /* xmlSecTransformUsage usage; */
+
+    xmlSecOpenSSLEvpDigestInitialize,           /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecOpenSSLEvpDigestFinalize,             /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    NULL,                                       /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    NULL,                                       /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecOpenSSLEvpDigestVerify,               /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecOpenSSLEvpDigestExecute,              /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecOpenSSLTransformSha3_512GetKlass:
+ *
+ * SHA3-512 digest transform klass.
+ *
+ * Returns: pointer to SHA3-512 digest transform klass.
+ */
+xmlSecTransformId
+xmlSecOpenSSLTransformSha3_512GetKlass(void) {
+    return(&xmlSecOpenSSLSha3_512Klass);
+}
+#endif /* XMLSEC_NO_SHA3 */
+
 #ifndef XMLSEC_NO_GOST
 /******************************************************************************
  *
@@ -915,4 +1122,3 @@ xmlSecOpenSSLTransformGostR3411_2012_512GetKlass(void) {
 }
 
 #endif /* XMLSEC_NO_GOST2012 */
-
