@@ -430,6 +430,17 @@ execEncTest $res_success \
     "--session-key aes-256 --xml-data $topfolder/aleksey-xmlenc-01/enc-two-enc-keys.data --pubkey-cert-$cert_format:key1 $topfolder/keys/cacert.$cert_format --pubkey-cert-$cert_format:key2 $topfolder/keys/ca2cert.$cert_format" \
     "$priv_key_option:key2 $topfolder/keys/ca2key.$priv_key_format --pwd secret123"
 
+
+execEncTest $res_success \
+    "" \
+    "aleksey-xmlenc-01/large_input" \
+    "aes256-cbc rsa-1_5" \
+    "x509" \
+    "$priv_key_option:my-key $topfolder/keys/largersakey.$priv_key_format --pwd secret123" \
+    "--session-key aes-256 --xml-data $topfolder/aleksey-xmlenc-01/large_input.data --pubkey-cert-$cert_format:my-key $topfolder/keys/largersacert.$cert_format" \
+    "$priv_key_option:my-key $topfolder/keys/largersakey.$priv_key_format --pwd secret123"
+
+
 execEncTest $res_success \
     "" \
     "aleksey-xmlenc-01/enc-des3cbc-keyname" \
