@@ -50,14 +50,14 @@ var withMSCrypto = 0;
 var withMSCng = 0;
 var withLibXSLT = 1;
 var withIconv = 1;
-var withFTP = 1;
-var withNT4 = 1;
-var withFTP = 0; /* disable ftp by default */
-var withHTTP = 0; /* disable http by default */
+var withSizeT = 1;
+var withLegacyCrypto = 0;
 
 /* Win32 build options. */
+var withNT4 = 0;
 var buildUnicode = 1;
 var buildDebug = 0;
+var buildWithMemcheck = 0;
 var buildWerror = 0;
 var buildPedantic = 1;
 var buildCc = "cl.exe";
@@ -118,13 +118,14 @@ function usage()
 	txt += "              \"openssl-110\", \"openssl=300\", \"openssl-300\",\n";
 	txt += "              \"nss\", \"mscrypto\", \"mscng\" (\"" + withCrypto + "\");\n"
  	txt += "  xslt:       LibXSLT is used (" + (withLibXSLT? "yes" : "no")  + ")\n";
-	txt += "  iconv:      Use the iconv library (" + (withIconv ? "yes" : "no") + ")\n";
-	txt += "  ftp:        Enable FTP support (" + (withFTP ? "yes" : "no") + ")\n";
-	txt += "  http:       Enable HTTP support (" + (withHTTP ? "yes" : "no") + ")\n";
- 	txt += "  nt4:        Enable NT 4.0 support (" + (withNT4? "yes" : "no")  + ")\n";
+ 	txt += "  iconv:      Use the iconv library (" + (withIconv? "yes" : "no")  + ")\n";
+	txt += "  size_t:     Use the size_t (" + (withSizeT ? "yes" : "no") + ")\n";
+	txt += "  legacy - crypto:  Use the size_t (" + (withLegacyCrypto ? "yes" : "no") + ")\n";
 	txt += "\nWin32 build options, default value given in parentheses:\n\n";
+	txt += "  nt4:        Enable NT 4.0 support (" + (withNT4 ? "yes" : "no") + ")\n";
 	txt += "  unicode:    Build Unicode version (" + (buildUnicode? "yes" : "no")  + ")\n";
 	txt += "  debug:      Build unoptimised debug executables (" + (buildDebug? "yes" : "no")  + ")\n";
+	txt += "  memcheck:   Build unoptimised debug executables with memcheck reporting (" + (buildWithMemcheck ? "yes" : "no") + ")\n";
 	txt += "  werror:     Build with warnings as errors (" + (buildWerror? "yes" : "no")  + ")\n";
     txt += "  pedantic:   Build with more warnings enabled (" + (buildPedantic? "yes" : "no") + ")\n";
 	txt += "  cc:         Build with the specified compiler(" + buildCc  + ")\n";
@@ -145,7 +146,7 @@ function usage()
 	txt += "  lib:        Additional search path for the linker, particularily\n";
 	txt += "              where libxml library can be found (" + buildLib + ")\n";
 	txt += "\nCrypto options, default value given in parentheses:\n\n";
-	txt += "  with-openssl3-engines:    Enable dynamic loading of xmlsec-crypto libraries (" + (withOpenSSL3Engines ? "yes" : "no") + ")\n";
+	txt += "  with-openssl3-engines:    Enable dynamic loading of xmlsec-crypto libraries (" + (withOpenSSL3Engines ? "yes" : "no") + ")\n";	
 	WScript.Echo(txt);
 }
 
@@ -192,11 +193,12 @@ function discoverVersion()
 	vf.WriteLine("WITH_MSCNG=" + withMSCng);
 	vf.WriteLine("WITH_LIBXSLT=" + (withLibXSLT ? "1" : "0"));
 	vf.WriteLine("WITH_ICONV=" + (withIconv ? "1" : "0"));
-	vf.WriteLine("WITH_FTP=" + (withFTP ? "1" : "0"));
-	vf.WriteLine("WITH_HTTP=" + (withHTTP ? "1" : "0"));
+	vf.WriteLine("WITH_SIZE_T=" + (withSizeT ? "1" : "0"));
+	vf.WriteLine("WITH_LEGACY_CRYPTO=" + (withLegacyCrypto ? "1" : "0"));
 	vf.WriteLine("WITH_NT4=" + (withNT4 ? "1" : "0"));
 	vf.WriteLine("UNICODE=" + (buildUnicode? "1" : "0"));
 	vf.WriteLine("DEBUG=" + (buildDebug? "1" : "0"));
+	vf.WriteLine("MEMCHECK=" + (buildWithMemcheck ? "1" : "0"));
 	vf.WriteLine("WERROR=" + (buildWerror? "1" : "0"));
 	vf.WriteLine("PEDANTIC=" + (buildPedantic? "1" : "0"));
 	vf.WriteLine("CC=" + buildCc);
@@ -314,6 +316,7 @@ function genReadme(bname, ver, file)
  */
 
 /* Parse the command-line arguments. */
+var cruntimeSet = 0
 for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 	var arg, opt;
 	arg = WScript.Arguments(i);
@@ -327,17 +330,20 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			withLibXSLT = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "iconv")
 			withIconv = strToBool(arg.substring(opt.length + 1, arg.length));
-		else if (opt == "ftp")
-			withFTP = strToBool(arg.substring(opt.length + 1, arg.length));
-		else if (opt == "http")
-			withHTTP = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "size_t")
+			withSizeT = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "legacy-crypto")
+			withLegacyCrypto = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "nt4")
 			withNT4 = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "unicode")
 			buildUnicode = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "debug")
 			buildDebug = strToBool(arg.substring(opt.length + 1, arg.length));
-		else if (opt == "werror")
+		else if (opt == "memcheck") {
+			buildWithMemcheck = strToBool(arg.substring(opt.length + 1, arg.length));
+			buildDebug = true;
+		} else if (opt == "werror")
 			buildWerror = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "pedantic")
 			buildPedantic = strToBool(arg.substring(opt.length + 1, arg.length));
@@ -365,9 +371,10 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			buildInclude = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "lib")
 			buildLib = arg.substring(opt.length + 1, arg.length);
-		else if (opt == "cruntime")
+		else if (opt == "cruntime") {
 			cruntime = arg.substring(opt.length + 1, arg.length);
-		else if (opt == "with-openssl3-engines")
+			cruntimeSet = 1;
+		} else if (opt == "with-openssl3-engines")
 			withOpenSSL3Engines = strToBool(arg.substring(opt.length + 1, arg.length));
 		else
 			error = 1;
@@ -381,9 +388,15 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			usage();
 			WScript.Quit(0);
 		}
-	} else
+	} else {
 		error = 1;
+	}
 }
+
+if (cruntimeSet == 0 && buildDebug != 0) {
+	cruntime = cruntime + "d";
+}
+
 // If we have an error here, it is because the user supplied bad parameters.
 if (error != 0) {
 	usage();
@@ -400,10 +413,6 @@ for (j = 0; j < crlist.length; j++) {
 		curcrypto="openssl";
 		withOpenSSL = 1;
 		withOpenSSLVersion = "110"; /* default */
-	} else if (crlist[j] == "openssl=100" || crlist[j] == "openssl-100") {
-		curcrypto="openssl";
-		withOpenSSL = 1;
-		withOpenSSLVersion = "100";
 	} else if (crlist[j] == "openssl=110" || crlist[j] == "openssl-110") {
 		curcrypto="openssl";
 		withOpenSSL = 1;
@@ -469,15 +478,16 @@ txtOut += "       Use MSCrypto: " + boolToStr(withMSCrypto) + "\n";
 txtOut += "          Use MSCng: " + boolToStr(withMSCng) + "\n";
 txtOut += "        Use LibXSLT: " + boolToStr(withLibXSLT) + "\n";
 txtOut += "          Use iconv: " + boolToStr(withIconv) + "\n";
-txtOut += "        Support FTP: " + boolToStr(withFTP) + "\n";
-txtOut += "       Support HTTP: " + boolToStr(withHTTP) + "\n";
-txtOut += "     NT 4.0 support: " + boolToStr(withNT4) + "\n";
+txtOut += "         Use size_t: " + boolToStr(withSizeT) + "\n";
+txtOut += "  Use legacy crypto: " + boolToStr(withLegacyCrypto) + "\n";
 txtOut += "\n";
 txtOut += "Win32 build configuration\n";
 txtOut += "-------------------------\n";
+txtOut += "     NT 4.0 support: " + boolToStr(withNT4) + "\n";
 txtOut += "  C-Runtime option: " + cruntime + "\n";
 txtOut += "           Unicode: " + boolToStr(buildUnicode) + "\n";
 txtOut += "     Debug symbols: " + boolToStr(buildDebug) + "\n";
+txtOut += "          Memcheck: " + boolToStr(buildWithMemcheck) + "\n";
 txtOut += "Warnings as errors: " + boolToStr(buildWerror) + "\n";
 txtOut += "          Pedantic: " + boolToStr(buildPedantic) + "\n";
 txtOut += "        C compiler: " + buildCc + "\n";
